/*
 *  linux/drivers/mmc/mtk_sd.c - Secure Digital Host Controller Interface driver
 *
 *  Copyright (C) 2009 Mediatek.Inc, All Rights Reserved.
 *
 */
#include <config.h>
#include <common.h>
#include <command.h>
#include <hwconfig.h>
#include <mmc.h>
#include <part.h>
#include <malloc.h>
#include <mmc.h>
#include <asm/io.h>
#include <asm/arch/x_typedef.h>
#include <asm/arch/mt85xx.h>

#include <mt85xx_gpio.h>

#include "mtk_msdc.h"
#include "msdchw_reg.h"
#include "msdchw_io.h"
#include "msdc_const.h"
#include "x_ckgen.h"


#define DRIVER_NAME "MTK_MSDC"
#define DRIVER_VERSION "0.10"

#define BUGMAIL "Mediatek. Inc"

DECLARE_GLOBAL_DATA_PTR;

/*------------------------------------------------------------------------------
    LOG Debug function
------------------------------------------------------------------------------*/
#ifdef MSDC_LOG_ENABLE
#define MSDC_LOG_ERROR                       ((u32)1 << 0)
#define MSDC_LOG_CARD                         ((u32)1 << 1)
#define MSDC_LOG_REQ                           ((u32)1 << 2)
#define MSDC_LOG_CLK                           ((u32)1 << 3)
#define MSDC_LOG_RETRY                       ((u32)1 << 4)
	
#define MSDC_LOG_INIT                          ((u32)1 << 5)
#define MSDC_LOG_FUNC                        ((u32)1 << 6)
#define MSDC_LOG_IRQ                           ((u32)1 << 8)
#define MSDC_LOG_SDHWCMD                 ((u32)1 << 9)
#define MSDC_LOG_IOCMD                       ((u32)1 << 10)
#define MSDC_LOG_RW                            ((u32)1 << 11)
#define MSDC_LOG_BUFFER                     ((u32)1 << 12)
#define MSDC_LOG_DBG                          ((u32)1 << 13)
	
static unsigned int msdc_log_en =  MSDC_LOG_RETRY | MSDC_LOG_ERROR | MSDC_LOG_CARD | MSDC_LOG_CLK;

#define MSDC_LOG(n, f, x...) if (msdc_log_en & (n)){ printf(DRIVER_NAME ": <%s	> " f, __func__,## x); }
//#define FCI_LOG(n, f, x...) if (fci_log_en & (n)){ printf(DRIVER_NAME ": <%s> " f, __func__,## x); }

#else
#define MSDC_LOG(n, f, x...)
#endif


/*------------------------------------------------------------------------------
    Static variables
------------------------------------------------------------------------------*/

static struct mtk_msdc_host msdc_host;

volatile unsigned int MsdcAccuVect = 0x00;

//static UINT32 u4msdcFreqIdx = 0;
#if ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580))
static UINT32 u4MsdcFreq[] = {INIT_BUS_FREQ, MSDC_CLK_3375K, MSDC_CLK_13_5M,MSDC_CLK_23M, MSDC_CLK_27M, BUS_CLK_FREQ};
#endif

/*------------------------------------------------------------------------------
    Global variables
------------------------------------------------------------------------------*/

BOOL _bFCICardExist = FALSE;

/*------------------------------------------------------------------------------
    Imported Global variables
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
    Module Parameter variable Entry
------------------------------------------------------------------------------*/
static INT32 _u4SDMultiFuncCfg = SD_MLT_FNC_CFG;
static INT32 _u4SDPwrCdPinCfg = SD_PWR_CD_PIN_CFG;
static INT32 _u4SDSdWpPinCfg = SD_WP_PIN_CFG;

/*------------------------------------------------------------------------------
    Static  functions
------------------------------------------------------------------------------*/
static int MsdcWaitClkStable (UINT32 ch);
static VOID MsdcSDDrivingCurrent (UINT32 ch,INT32 i4ClkMHz);
static VOID CKGEN_AgtSelClk(UINT32 ch, UINT32 u4Sel);


/*-------------------------g-----------------------------------------------------
    Imported Global functions
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
    MSDC base functions
------------------------------------------------------------------------------*/
// ---------------------------------
// Return BOOL : TRUE - card exist
// Return BOOL : FALSE - card not exist
// ---------------------------------
int msdc_card_exist(void)
{
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)

  if (_u4SDPwrCdPinCfg == 1)
  {
	GPIO_Config(74, 0, 1);  //config PIN_VOUT7 as input mode
	MSDC_LOG(MSDC_LOG_FUNC, "card_exist: %d \n", GPIO_Input(74));
	return ((GPIO_Input(74) ? 0 : 1));
  }
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
	if (_u4SDPwrCdPinCfg == 1)
	{
	  GPIO_Config(10, 0, 1);  //config PIN_VOUT7 as input mode
	  return ((GPIO_Input(10) ? FALSE : TRUE));
	}
#endif

  return FALSE;
}

//-------------------------------------------------------------------------
/** MsdcSetClkfreq
 *  Set MSDC clock divide
 *  @param  ch : channel
 *  @param  clkFreq : Target frequency setting (in Hz)
 *  @return  void
 */
//-------------------------------------------------------------------------
static int MsdcSetClkfreq (UINT32 ch, unsigned int clkFreq)
{
 // unsigned int divider;
 // unsigned int expFreq = 0;
/*  
  if ((clkFreq > BUS_CLK_FREQ))
  {
    clkFreq = BUS_CLK_FREQ;
    MSDC_LOG(MSDC_LOG_ERROR, "%s: over max frequency setting, set clkFreq : %d Hz\r\n", __FUNCTION__, clkFreq);
  }
  else if ((clkFreq < LOWEST_BUS_FREQ))
  {
    clkFreq = INIT_BUS_FREQ;
    MSDC_LOG(MSDC_LOG_ERROR, "%s: set clkFreq : %d Hz\r\n", __FUNCTION__, clkFreq);
  }
*/
#if ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580))
  CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL1_1_8);
  x_thread_delay(2);
  
  switch (clkFreq)
  {
    case MSDC_CLK_54M:
   //   _u4SDClockSetting = 54;
      // 0x7000_0078 bit[14:8] = xxx_0001, SD_CLK = 54 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL1_1_8);
      break;
  
    case MSDC_CLK_50M:
  //    _u4SDClockSetting = 50;
      // 0x7000_0078 bit[14:8] = xxx_0100, SD_CLK = 50 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL2_1_12);
      break;
  
    case MSDC_CLK_36M:
     // _u4SDClockSetting = 36;
      // 0x7000_0078 bit[14:8] = xxx_0011, SD_CLK = 36 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL1_1_12);
      break;
			  
    case MSDC_CLK_27M:
    //  _u4SDClockSetting = 27;
      // 0x7000_0078 bit[14:8] = xxx_0000, SD_CLK = 27 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M);
      break;

    case MSDC_CLK_23M:
    //  _u4SDClockSetting = 23;
      // 0x7000_0078 bit[14:8] = xxx_0111, SD_CLK = 23.08 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL2_1_26);
      break;
						  
    case MSDC_CLK_13_5M:
     // _u4SDClockSetting = 13;
      // 0x7000_0078 bit[14:8] = 000_1001, SD_CLK = 13.50 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M_1_2);
      break;   	   
			  
    case MSDC_CLK_3375K:
     // _u4SDClockSetting = 3;
      //  0x7000_0078 bit[14:8] = 010_1001, SD_CLK = 3.38 MHz  OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M_1_8);
      break; 	   
			  
    case MSDC_CLK_420K:
    default:
     // _u4SDClockSetting = 0;
      //  0x7000_0078 bit[14:8] = 101_1001, SD_CLK = 421.885 KHz  OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M_1_64);
      // 0x7000_0078 bit[14:8] = 110_1001, SD_CLK = 210.94 KHz	OK
     //	CKGEN_AgtSelClk(e_CLK_SD, CLK_CLK_SD_SEL_27M_1_128);
     break;
  
     // For reference
     // 0x7000_0078 bit[14:8] = 100_1001, SD_CLK = 843.75 KHz	OK
     // 0x7000_0078 bit[14:8] = 110_1001, SD_CLK = 210.94 KHz	OK
     // 0x7000_0078 bit[14:8] = 111_1001, SD_CLK = 105.47 KHz	OK

  } 
  x_thread_delay(1);
#endif
//  MSDC_WRITE32(MSDC_CFG(ch), (MSDC_READ32(MSDC_CFG(ch)) & (~0x0000FF00)));

  // Wait until clock is stable
  if (FAILED == MsdcWaitClkStable(ch))
  {
     return FAILED;
  }
  
  MSDC_LOG(MSDC_LOG_CLK, "Clock set OK! Clk Frequency : %d Hz\r\n", clkFreq);
  
  // Setup driving current according to working frequency
  MsdcSDDrivingCurrent (ch, clkFreq/1000000);
  
  msdc_host.clk = clkFreq;
  
  return SUCCESS;


/*
  for (divider = 0; divider <= 0xFF; divider++)
  {
    if (0 == divider)
    {
      expFreq = SRC_CLK_FREQ / 2;
    }
    else
    {
      expFreq = SRC_CLK_FREQ / 4 / divider;
    }

    if (expFreq <= clkFreq)
    {
      MSDC_WRITE32(MSDC_CFG(ch), (MSDC_READ32(MSDC_CFG(ch)) & (~0x0000FF00)) | (divider << MSDC_CFG_CK_DIV_SHIFT));

      // Wait until clock is stable
      if (FAILED == MsdcWaitClkStable(ch))
      {
        return FAILED;
      }

      MSDC_LOG(MSDC_LOG_CLK, "%s: Divider 0x%02X, clkFreq : %d, Expect Frequency : %d Hz\r\n", __FUNCTION__, divider, clkFreq, expFreq);

      // Setup driving current according to working frequency
      MsdcSDDrivingCurrent (expFreq/1000);

      msdc_host->clk = clkFreq;
      
      return SUCCESS;
    }
  }
*/
  MSDC_LOG(MSDC_LOG_ERROR, "Failed to search correct divider.\r\n");

  return FAILED;
}


static void msdc_set_clock(struct mtk_msdc_host *host, UINT32 dest_clk)
{
  unsigned int clk;

  UINT32 i, level;

  level = sizeof(u4MsdcFreq) /sizeof(UINT32);
  for(i=0; i<level; i++)
  {
    if(dest_clk <= u4MsdcFreq[0])
  {
    clk = u4MsdcFreq[0];
       break;
    }
    else if(dest_clk >= u4MsdcFreq[level-1])
    {
       clk = u4MsdcFreq[level-1];
       break;
    }
    else if(dest_clk < u4MsdcFreq[i])
    {
       clk = u4MsdcFreq[i-1];
       break;
    }
  }

  // Only Test: clock is always output
 // MsdcContinueClock(0, 1);
	
  MSDC_LOG(MSDC_LOG_CLK, "level=%d, Dest_Clk = %dhz, set_clk = %d hz\n", level, dest_clk, clk);

 /*
   * Only write to the clock register when
   * there is an actual change.
   */
  if (clk != host->clk)
  {
    MsdcSetClkfreq(0, clk);
  }
}

//-------------------------------------------------------------------------
/** MsdcSetBusWidth
 *  Msdc hardware bus width setting function
 *  @param  ch : channel
 *  @param  busWidth : 1 or 4 bit
 *  @return  void
 */
//-------------------------------------------------------------------------
static int MsdcSetBusWidth (UINT32 ch, int busWidth)
{
  /* Modify MSDC Register Settings */
  if (1 == busWidth)
  {
    MSDC_WRITE32(SDC_CFG(ch), (MSDC_READ32(SDC_CFG(ch)) & ~SDC_CFG_BW_MASK) | SDC_CFG_BW_1BIT);
  }
  else if (4 == busWidth)
  {
    MSDC_WRITE32(SDC_CFG(ch), (MSDC_READ32(SDC_CFG(ch)) & ~SDC_CFG_BW_MASK) | SDC_CFG_BW_4BIT);
  }

  return SUCCESS;
}

static void msdc_set_ios(struct mmc *mmc)
{
  int ch = 0;
   
  /* Set the clock speed */	
  msdc_set_clock(mmc->priv, mmc->clock);


  /* Set the bus width */
  if ( msdc_host.bus_width != mmc->bus_width)
  {
    msdc_host.bus_width = mmc->bus_width;
		
    MSDC_LOG(MSDC_LOG_CARD, "Switch to %d Bit Mode.\n", mmc->bus_width); 
    MsdcSetBusWidth(ch, mmc->bus_width);
  }
}


/*------------------------------------------------------------------------------
    MSDC Low Level functions
------------------------------------------------------------------------------*/
//-------------------------------------------------------------------------
/** MsdcReset
 *  Reset Msdc HW (register value will not be reset)
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int MsdcReset (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  // Reset MSDC
  MSDC_SETBIT(MSDC_CFG(ch), MSDC_CFG_RST);

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (0 == (MSDC_READ32(MSDC_CFG(ch)) & MSDC_CFG_RST))
    {
      break;
    }
    else if (u4DiffTime >= MSDC_RST_TIMEOUT_LIMIT)
    {
      MSDC_LOG(MSDC_LOG_ERROR, "timeout error !\n");
      return FAILED;
    }

    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}

//-------------------------------------------------------------------------
/** MsdcClrFifo
 *  Clear Msdc FIFO status.
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int MsdcClrFifo (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

//  MSDC_WAIT_SETBIT(MSDC_FIFOCS(ch), MSDC_FIFOCS_FIFOCLR);

  // Reset FIFO
  MSDC_SETBIT(MSDC_FIFOCS(ch), MSDC_FIFOCS_FIFOCLR);

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (0 == (MSDC_READ32(MSDC_FIFOCS(ch)) & (MSDC_FIFOCS_FIFOCLR | MSDC_FIFOCS_TXFIFOCNT_MASK | MSDC_FIFOCS_RXFIFOCNT_MASK)))
    {
      break;
    }
    else if (u4DiffTime >= MSDC_FIFOCLR_TIMEOUT_LIMIT)
    {
      MSDC_LOG(MSDC_LOG_ERROR, "timeout error !\n");
      return FAILED;
    }

    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}

//-------------------------------------------------------------------------
/** MsdcChkFifo
 *  Check if Msdc FIFO = 0.
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static void MsdcChkFifo (UINT32 ch)
{
  // Check if rx/tx fifo is zero
  if ((MSDC_READ32(MSDC_FIFOCS(ch)) & (MSDC_FIFOCS_TXFIFOCNT_MASK | MSDC_FIFOCS_RXFIFOCNT_MASK)) != 0)
  {
    MSDC_LOG(MSDC_LOG_ERROR, "FiFo not 0, FIFOCS:0x%08X !!\n", MSDC_READ32(MSDC_FIFOCS(ch)));
    MsdcClrFifo(ch);
  }
}

//-------------------------------------------------------------------------
/** MsdcClrIntr
 *  Clear MSDC interrupt vector
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static void MsdcClrIntr (UINT32 ch)
{
  // Check MSDC Interrupt vector register
  if  (0x00  != MSDC_READ32(MSDC_INT(ch)))
  {
    MSDC_LOG(MSDC_LOG_ERROR, "MSDC INT not 0:0x%08X !!\r\n", MSDC_READ32(MSDC_INT(ch)));

    // Clear MSDC Interrupt vector register
    MSDC_WRITE32(MSDC_INT(ch), MSDC_READ32(MSDC_INT(ch)));
  }
}


//-------------------------------------------------------------------------
/** MsdcWaitClkStable
 *  Wait MSDC Clock stable after Mode/DIV changed
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int MsdcWaitClkStable (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms
    if (0 != (MSDC_READ32(MSDC_CFG(ch)) & MSDC_CFG_CARD_CK_STABLE))
    {
      break;
    }
    else if (u4DiffTime >= MSDC_CLK_TIMEOUT_LIMIT)
    {
      MSDC_LOG(MSDC_LOG_ERROR, "WaitClkStable Failed !\r\n");
      return FAILED;
    }
    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}

//-------------------------------------------------------------------------
/** MsdcContinueClock
 *  Set continue clock output or not
 *  @param  ch : channel
 *  @param  i4ContinueClock TRUE - Continue clock, FALSE - clock output when command is sending
 *  @return  void
 */
//-------------------------------------------------------------------------
static void MsdcContinueClock (UINT32 ch, int i4ContinueClock)
{
  if (i4ContinueClock)
  {
    // Set clock continuous even if no command
    MSDC_SETBIT(MSDC_CFG(ch), (((UINT32)0x01) << 1));
  }
  else
  {
    // Set clock power down if no command
    MSDC_CLRBIT(MSDC_CFG(ch), (((UINT32)0x01) << 1));
  }
}

//-------------------------------------------------------------------------
/** MsdcHWConfigSetup
 *  Setup multi-func, driving current, pull up resistor ... HW setting.
 *  @return  VOID.
 */
//-------------------------------------------------------------------------
static VOID MsdcHWConfigSetup(void)
{
#if ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580))

#endif
}

//-------------------------------------------------------------------------
/** MsdcSDDrivingCurrent
 *  Set SD driving current according to clock
 *  @param i4ClkMHz    clock setting (Mhz).
 *  @return  VOID
 */
//-------------------------------------------------------------------------

static VOID MsdcSDDrivingCurrent (UINT32 ch,INT32 i4ClkMHz)
{
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
  switch(_u4SDMultiFuncCfg)
  {
    case 1:
      {
        // Driving current setting
        if (i4ClkMHz >= 27000000)
        {
          // Set driving current E4+E8
        }
        else
        {
          // Set driving current E2
        }
      }
      break;
  }
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
    MSDC_LOG(MSDC_LOG_CLK, "set driver current: %d Hz\r\n", i4ClkMHz);
	if (i4ClkMHz >= 27)
	{
	  // Set driving current E4+E8 
	  MSDC_LOG(MSDC_LOG_CLK, "set driver current to max\r\n");
	  MSDC_WRITE32(SD20_PAD_CTL0(ch), ((MSDC_READ32(SD20_PAD_CTL1(ch)) & 0xFFFFFFFC) | 0x3));// Set clock driving current E4+E8
	  MSDC_WRITE32(SD20_PAD_CTL1(ch), ((MSDC_READ32(SD20_PAD_CTL1(ch)) & 0xFFFFFFFC) | 0x3));// Set cmd driving current E4+E8
	  MSDC_WRITE32(SD20_PAD_CTL2(ch), ((MSDC_READ32(SD20_PAD_CTL2(ch)) & 0xFFFFFFFC) | 0x3));// Set data driving current E4+E8
	}
	else
	{
	  // Set driving current E2
	  MSDC_LOG(MSDC_LOG_CLK, "set driver current to min\r\n");
	  MSDC_WRITE32(SD20_PAD_CTL0(ch), (MSDC_READ32(SD20_PAD_CTL1(ch)) & 0xFFFFFFFC));// Set clock driving current E4+E8
	  MSDC_WRITE32(SD20_PAD_CTL1(ch), (MSDC_READ32(SD20_PAD_CTL1(ch)) & 0xFFFFFFFC));// Set cmd driving current E4+E8
	  MSDC_WRITE32(SD20_PAD_CTL2(ch), (MSDC_READ32(SD20_PAD_CTL2(ch)) & 0xFFFFFFFC));// Set data driving current E4+E8
	}
#endif
}

//-------------------------------------------------------------------------
/** CKGEN_AgtEnableClk
 *  Turn on / off SD dram clock
 *  @param fgEnable    on / off
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID CKGEN_AgtEnableClk(UINT32 ch, BOOL fgEnable)
{
  UINT32 u4Tmp;

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) 
  u4Tmp = CKGEN_READ32(REG_RW_CLK_CFG2);

  if(fgEnable)
    u4Tmp = u4Tmp & (~CLK_PDN_SD);
  else
    u4Tmp = u4Tmp | CLK_PDN_SD;
  
  CKGEN_WRITE32(REG_RW_CLK_CFG2, u4Tmp);
  MSDC_LOG(MSDC_LOG_ERROR, "Enable MSDC dram Clk !\n");
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)  
  u4Tmp = CKGEN_READ32(REG_RW_CLK_CFG7);

  if(fgEnable)
	u4Tmp = u4Tmp & (~CLK_PDN_SD);
  else
	u4Tmp = u4Tmp | CLK_PDN_SD;

  CKGEN_WRITE32(REG_RW_CLK_CFG7, u4Tmp);
  
#endif
	x_thread_delay(1);
    return;
}

//-------------------------------------------------------------------------
/** CKGEN_AgtSelClk
 *  set dram clock
 *  @param u4Sel : offset
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID CKGEN_AgtSelClk(UINT32 ch, UINT32 u4Sel)
{
    UINT32 u4Tmp;
    
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) 
  u4Tmp = CKGEN_READ32(REG_RW_CLK_CFG2);

  u4Sel = (u4Sel << CLK_CLK_SD_SEL_OFFSET) & CLK_CLK_SD_SEL_MASK;
  u4Tmp = u4Tmp & (~CLK_CLK_SD_SEL_MASK);
  u4Tmp = u4Tmp | u4Sel;
              
  CKGEN_WRITE32(REG_RW_CLK_CFG2, u4Tmp);
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580) 
  u4Tmp = CKGEN_READ32(REG_RW_CLK_CFG7);

  u4Sel = (u4Sel << CLK_CLK_SD_SEL_OFFSET) & CLK_CLK_SD_SEL_MASK;
  u4Tmp = u4Tmp & (~CLK_CLK_SD_SEL_MASK);
  u4Tmp = u4Tmp | u4Sel;
			  
  CKGEN_WRITE32(REG_RW_CLK_CFG7, u4Tmp);

#endif


    return;    
}



//-------------------------------------------------------------------------
/** MsdcInit
 *  Msdc hardware Initialize function
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int MsdcInit (UINT32 ch)
{
  CKGEN_AgtEnableClk(ch, TRUE);

  // Reset MSDC
  MsdcReset(ch);

  // Set DMA Mode & SD/MMC Mode  
  MSDC_CLRBIT(MSDC_CFG(ch), MSDC_CFG_PIO_MODE);
  MSDC_SETBIT(MSDC_CFG(ch), MSDC_CFG_DMA_MODE | MSDC_CFG_SD);

  // Disable sdio & Set bus to 1 bit mode
  MSDC_CLRBIT(SDC_CFG(ch), SDC_CFG_SDIO | SDC_CFG_BW_MASK);

  // set clock mode (Direct mode)
  MSDC_CLRBIT(MSDC_CFG(ch), (((UINT32)0x03) << 16));
  MSDC_SETBIT(MSDC_CFG(ch), MSDC_CFG_CK_MODE_DIRECT);

  // Wait until clock is stable
  if (FAILED == MsdcWaitClkStable(ch))
  {
    return FAILED;
  }

  // Set default RISC_SIZE for DWRD pio mode
  MSDC_WRITE32(MSDC_IOCON(ch), (MSDC_READ32(MSDC_IOCON(ch)) & ~MSDC_IOCON_RISC_SIZE_MASK) | MSDC_IOCON_RISC_SIZE_DWRD);

  // Set Data timeout setting => Maximum setting
  MSDC_WRITE32(SDC_CFG(ch), (MSDC_READ32(SDC_CFG(ch)) & ~(((UINT32)0xFF) << SDC_CFG_DTOC_SHIFT)) | (((UINT32)0xFF) << SDC_CFG_DTOC_SHIFT));

  // Enable interrupt bit
  MSDC_SETBIT(MSDC_INTEN(0), INT_SD_ALL);

  // Setup HW configuration for MSDC
  MsdcHWConfigSetup();
  
  return SUCCESS;
}



//-------------------------------------------------------------------------
/** MsdcPowerOn
 *  Power on MSDC Clock
 *  @param  ch : channel
 *  @return TRUE.
 */
//-------------------------------------------------------------------------
static void MsdcPowerOn (UINT32 ch)
{
  // Card CK Drive enable
  MSDC_SETBIT(MSDC_CFG(ch), MSDC_CFG_CK_DRV_EN);

  CKGEN_AgtEnableClk(ch, TRUE);
  
}

//-------------------------------------------------------------------------
/** MsdcPowerOff
 *  Power off MSDC Clock
 *  @param  ch : channel
 *  @return TRUE.
 */
//-------------------------------------------------------------------------
static void MsdcPowerOff (UINT32 ch)
{
  // Card CK Drive enable
  MSDC_CLRBIT(MSDC_CFG(ch), MSDC_CFG_CK_DRV_EN);

  CKGEN_AgtEnableClk(ch, FALSE);
}

//-------------------------------------------------------------------------
/** MsdcDmaEnable
 *  Enable MSDC DMA for read/write operation
 *  @param ch            channel
 *  @param pu1Buf      user's read buffer pointer.
 *  @param u4BlkSz     write block size in bytes.
 *  @param fgWrite     TRUE for write, FALSE for read.
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID MsdcDmaEnable (UINT32 ch, UINT8 *pu1Buf, UINT32 u4BlkSz, BOOL fgWrite)
{
  ASSERT(pu1Buf);

//  BSP_FlushDCacheRange((UINT32)pu1Buf, u4BlkSz);

  MSDC_WRITE32(DMA_SA(ch), (UINT32) pu1Buf);
  MSDC_WRITE32(DMA_CTRL(ch), (u4BlkSz << DMA_CTRL_XFER_SIZE_SHIFT) | DMA_CTRL_BST_64 | DMA_CTRL_LAST_BUF | DMA_CTRL_START);
}

//-------------------------------------------------------------------------
/** MsdcHw_DmaDisable
 *  Disable DMA transfer.
 *  @param  ch : channel
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID MsdcDmaDisable (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  MSDC_SETBIT(DMA_CTRL(ch), DMA_CTRL_STOP);

  u4JiffSt = u4JiffEnd = get_timer(0);
  
   while (1)
   {
	  u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;	// unit : ms
  
	  if (! (MSDC_READ32(DMA_CTRL(ch)) & DMA_CTRL_STOP))
	  {
		break;
	  }
	  else if (u4DiffTime >= MSDC_WAIT_CMD_TIMEOUT_LIMIT)
	  {
		MSDC_LOG(MSDC_LOG_ERROR, "timeout error !\n");
		return FAILED;
	  }
  
	  u4JiffEnd = get_timer(0);
  }
 // while (MSDC_READ32(DMA_CTRL(ch)) & DMA_CTRL_STOP);
}

//-------------------------------------------------------------------------
/** MsdcDmaStatusChk
 *  Check DMA is in-active, for ever loop until DMA become in-active
 *  @param  ch : channel
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID MsdcDmaStatusChk (UINT32 ch)
{
  while (MSDC_READ32(DMA_CFG(ch)) & DMA_CFG_DMA_STATUS);
}

//-------------------------------------------------------------------------
/** MsdcErrorHandling
 *  Msdc Error handling function
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static void MsdcErrorHandling (UINT32 ch, struct mmc_data *data)
{
  MSDC_LOG(MSDC_LOG_ERROR, "1 \n" );
  // Step 1 : Reset MSDC by writing 1 to MSDC_CFG.RST and wait it becomes 0
  MsdcReset(ch);
  
  if (data)
  {
     MSDC_LOG(MSDC_LOG_ERROR, "2 \n" );
    // Step 2 : STOP DMA by writing 1 to DMA_CTRL.DMA_STOP and wait it becomes 0
    MsdcDmaDisable(ch);
	MSDC_LOG(MSDC_LOG_ERROR, "3 \n" );

    // Step 3 : Check DMA status is 0 by reading DMA_CS.DMA_STATUS
    MsdcDmaStatusChk(ch);    
  }
  MSDC_LOG(MSDC_LOG_ERROR, "4 \n" );
  
   MSDC_LOG(MSDC_LOG_ERROR, "5 \n" );
  // Step 4 : Clear FIFO by writing 1 to MSDC_FIFOCS.FIFOCLR and wait it becomes 0
  MsdcClrFifo(ch);
  MSDC_LOG(MSDC_LOG_ERROR, "6 \n" );

  // Step 5 : Check MSDC_FIFOCS.TXFIFOCNT/ RXFIFOCNT are both 0
  MsdcChkFifo(ch);
  MSDC_LOG(MSDC_LOG_ERROR, "7 \n" );

  // Step 6 : Make sure the MSDC_INT are all cleared
  MsdcClrIntr(ch);
}

//-------------------------------------------------------------------------
/** MsdcWaitSDCBusIdle
 *  Wait SD Bus Idle
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int MsdcWaitSDCBusIdle (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;
  int ret = SUCCESS, retry = 0;
  
  retry_waitIdle:
  // Wait SDC bus idle
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms
    if ((0 == (MSDC_READ32(SDC_STS(ch)) & (SDC_STS_SDCBUSY | SDC_STS_CMDBUSY))) && (0x00  == MSDC_READ32(MSDC_INT(ch))))
    {
      ret = SUCCESS;
      break;
    }
    else if (u4DiffTime >= MSDC_WAIT_SDC_BUS_TIMEOUT_LIMIT)
    {
      MSDC_LOG(MSDC_LOG_ERROR, "SDC_STS=0x%x, MSDC_INT=0x%x \n", MSDC_READ32(SDC_STS(ch)), MSDC_READ32(MSDC_INT(ch)));

	ret = FAILED;
	retry ++;
	break;
    }
    u4JiffEnd = get_timer(0);
  }
  
  if(retry==1)
  {	
     retry++;
     MsdcErrorHandling(ch, 0);
     
     goto retry_waitIdle;
  }
  return ret;
}


//-------------------------------------------------------------------------
/** MsdcPowerSwitch
 *  Initailize all Msdc card.
 *  @param  fgPowerOn TRUE : power on, FALSE : power off
 *  @retval   none
 */
//-------------------------------------------------------------------------
static VOID MsdcPowerSwitch(BOOL fgPowerOn, UINT32 u4Delayms)
{
  UNUSED(fgPowerOn);

 // msdc_host.fgPowerSwitchOn = fgPowerOn;

  MSDC_LOG(MSDC_LOG_RETRY, "mmc : Power %s for %d ms.\n", fgPowerOn ? "On" : "Off", u4Delayms);

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)

  if (_u4SDPwrCdPinCfg == 1)
  {
	 if(fgPowerOn)    // configure GPIO low to turn on FCI power
	 {
		GPIO_Config(74, 1, 0);  //config PIN_VOUTD7 as Output mode = LOW
		GPIO_Config(74, 0, 0);  //config PIN_VOUTD7 as input mode as card detect mode
	}
	else
	{
		GPIO_Config(74, 1, 1);  //config PIN_VIND30 as Output mode = High
	}
  }
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
	if (_u4SDPwrCdPinCfg == 1)
	{
	  if(fgPowerOn)	// configure GPIO low to turn on FCI power
	   {
		  GPIO_Config(10, 1, 0);  //config PIN_VOUTD7 as Output mode = LOW
		  GPIO_Config(10, 0, 0);  //config PIN_VOUTD7 as input mode as card detect mode
	  }
	  else
	  {
		  GPIO_Config(10, 1, 1);  //config PIN_VIND30 as Output mode = High
	  }
	}
#endif
// Apply delay time
x_thread_delay(u4Delayms);

}

// Return when any interrupt is matched or timeout
static int MsdcWaitIntr (unsigned int vector, unsigned int or, unsigned int timeoutCnt) 
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime=0;
  int ch = 0;
  
  u4JiffSt = u4JiffEnd = get_timer(0);  
  
  while (u4DiffTime <= timeoutCnt)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (or) 
    {
        // Status match any bit
        if (0 != (MSDC_READ32(MSDC_INT(ch)) & vector)) 
        {
            return SUCCESS;
        }
    }
    else 
    {
        // status exactly match
        if (vector == MSDC_READ32(MSDC_INT(ch)))
        {
            return SUCCESS;
        }
    }        
        
    u4JiffEnd = get_timer(0);
  } 

  // Timeout case
  return FAILED;
}

//-------------------------------------------------------------------------
/** MsdcWaitCmdRdyOrTo
 *  Wait command line ready. Use in commad write operation.
 *  @param  ch : channel
 *  @retval  S_OK  Success.
 *  @retval  Others  Fail.
 */
//-------------------------------------------------------------------------
static INT32 MsdcWaitCmdRdyOrTo(UINT32 ch, struct mtk_msdc_host *host, struct mmc_cmd *cmd)
{
    UINT32 u4CmdDoneVect, MsdcAccuVect;

#ifdef CC_CARD_DETECT_PIN
     // handle interrupt status only in card inserted state.
    if (!(msdc_card_exist()))
    {
        MSDC_LOG(MSDC_LOG_ERROR, " No Card error.\n");
        return E_NO_CARD;
    }
#endif

    // Wait for command and response if existed
    u4CmdDoneVect = INT_SD_CMDRDY | INT_SD_CMDTO | INT_SD_RESP_CRCERR;

    if (SUCCESS != MsdcWaitIntr(u4CmdDoneVect, 1, MSDC_WAIT_CMD_TIMEOUT_LIMIT)) 
    {
        MSDC_LOG(MSDC_LOG_ERROR, "Failed to send CMD/RESP, DoneVect = 0x%x.\r\n", u4CmdDoneVect);
        return E_NO_RESPONSE;
    }

    MsdcAccuVect = MSDC_READ32(MSDC_INT(ch));
    MSDC_WRITE32(MSDC_INT(ch), MSDC_READ32(MSDC_INT(ch)));
//    MsdcClrIntr(ch);

    MSDC_LOG(MSDC_LOG_IRQ, "MsdcAccuVect = 0x%x, MSDC_INT = 0x%x.\r\n", MsdcAccuVect, MSDC_READ32(MSDC_INT(ch)));
    if (MsdcAccuVect & INT_SD_CMDTO)
    {
        MSDC_LOG(MSDC_LOG_ERROR, "CMD%d ARG 0x%08X - CMD Timeout (AccuVect 0x%08X INTR 0x%08X).\r\n", cmd->cmdidx, cmd->cmdarg, MsdcAccuVect, MSDC_READ32(MSDC_INT(ch)));        
        return E_CMD_TIMEOUT;
    }
    else if (MsdcAccuVect & INT_SD_RESP_CRCERR)
    {
        MSDC_LOG(MSDC_LOG_ERROR, "CMD%d ARG 0x%08X - CMD CRC Error (AccuVect 0x%08X INTR 0x%08X).\r\n", cmd->cmdidx, cmd->cmdarg, MsdcAccuVect, MSDC_READ32(MSDC_INT(ch)));
        return E_CMD_RSP_CRC_ERR;
    }
    else if ((MsdcAccuVect & (~(INT_SD_CMDRDY))) || (0 != MSDC_READ32(MSDC_INT(ch))))
    {
        MSDC_LOG(MSDC_LOG_ERROR, "CMD%d ARG 0x%08X - UnExpect status (AccuVect 0x%08X INTR 0x%08X).\r\n", cmd->cmdidx, cmd->cmdarg, MsdcAccuVect, MSDC_READ32(MSDC_INT(ch)));
        return E_FAIL;
    }

    return S_OK;
}

//-------------------------------------------------------------------------
/** MsdcReadResp
 *  read SD response.
 *  @param  ch : channel
 *  @param index    response index, range in 1~4..
 *  @return  response value.
 */
//-------------------------------------------------------------------------
static UINT32 MsdcReadResp(UINT32 ch, INT32 index)
{
  UINT32 u4Val;
  
  switch(index)
  {
    case 2:
      u4Val = MSDC_READ32(SDC_RESP1(ch));
      break;
    case 3:
      u4Val = MSDC_READ32(SDC_RESP2(ch));
      break;
    case 4:
      u4Val = MSDC_READ32(SDC_RESP3(ch));
      break;
    case 1:
    default:
      u4Val = MSDC_READ32(SDC_RESP0(ch));
      break;
  }
  return (u4Val);
}

//-------------------------------------------------------------------------
/** MsdcWaitDataRdyOrTo
 *  Wait data line ready. Use in data read operation.
 *  @param  ch : channel
 *  @retval  S_OK  Success.
 *  @retval  Others  Fail.
 */
//-------------------------------------------------------------------------
static INT32 MsdcWaitDataRdyOrTo(UINT32 ch, struct mtk_msdc_host *host, struct mmc_data *data)
{
  UINT32 u4DataDoneVect, MsdcAccuVect;
  INT32 ret = S_OK;
#ifdef CC_CARD_DETECT_PIN
     // handle interrupt status only in card inserted state.
  if (!(msdc_card_exist()))
  {
    MSDC_LOG(MSDC_LOG_ERROR, " No Card error.\n");
    return E_NO_CARD;
  }
#endif  

  // Wait for command and response if existed
  u4DataDoneVect = INT_SD_DATA_STS;
  
  if (SUCCESS != MsdcWaitIntr(u4DataDoneVect, 1, MSDC_WAIT_DATA_TIMEOUT_LIMIT)) 
  {
    MSDC_LOG(MSDC_LOG_ERROR, "Failed to wait data IRQ, MSDC_INT = 0x%x.\r\n", MSDC_READ32(MSDC_INT(ch)));
    return E_DATA_TIMEOUT;
  }
  
  MsdcAccuVect = MSDC_READ32(MSDC_INT(ch));
  MSDC_WRITE32(MSDC_INT(ch), MSDC_READ32(MSDC_INT(ch)));
  //	MsdcClrIntr(ch);
  MSDC_LOG(MSDC_LOG_IRQ, "MsdcAccuVect = 0x%x, MSDC_INT = 0x%x.\r\n", MsdcAccuVect, MSDC_READ32(MSDC_INT(ch)));

  if (MsdcAccuVect & INT_SD_DATTO)
  {
    ret = E_DATA_TIMEOUT;
  }
  else if (MsdcAccuVect & INT_SD_DATA_CRCERR)
  {
    ret = E_DATA_CRC_ERROR;
  }
  else if (MsdcAccuVect & (INT_SD_XFER_COMPLETE|INT_DMA_XFER_DONE))
  {
    return S_OK;
  }
  else
  {    
    ret = E_FAIL;
  }  
  
  MSDC_LOG(MSDC_LOG_ERROR, "Data error: MSDC_INT=0x%x, MsdcAccuVect=0x%x\n", MSDC_READ32(MSDC_INT(ch)), MsdcAccuVect);
  
  return ret;        
}

//-------------------------------------------------------------------------
/** MsdcSendCmd
 *  Send SD command and argument.
 *  @param u4Cmd    SD command.
 *  @param u4Arg      SD argument.
 *  @retval  S_OK  Success.
 *  @retval  Others  Fail.
 */
//-------------------------------------------------------------------------
static INT32 MsdcSendCmd(UINT32 ch, struct mtk_msdc_host *host, UINT32 u4Cmd, UINT32 u4Arg)
{
  // Save for check
  host->u4Cmd = u4Cmd;
  host->u4Arg = u4Arg;
  host->u4CmdError = 0;
  
  // Clear Vector variable
//  MsdcAccuVect = 0;    
  MsdcClrIntr(ch);
  
  // Set command state flag
  host->sdCmdState = MSDC_SD_CMD_START;

  // Set SDC Argument
  MSDC_WRITE32(SDC_ARG(ch), u4Arg);

  /* Send the commands to the device */
  MSDC_WRITE32(SDC_CMD(ch), u4Cmd);

  return 0;
}




/*****************************************************************************\
 *                                                                           *
 * Core functions                                                            *
 *                                                                           *
\*****************************************************************************/



static int msdc_prepare_data(struct mtk_msdc_host *host, struct mmc_data *data)
{
  int ret = S_OK;
  unsigned int size;

  /*
   * Calculate size.
   */
  size = data->blocks * data->blocksize;

  /*
   * The buffer for DMA is only 64 kB.
   */
  ASSERT(size < BASIC_DMA_MAX_LEN);

  if (size > BASIC_DMA_MAX_LEN)
  {
    ret = E_INVALID_ARG;
    MSDC_LOG(MSDC_LOG_ERROR, "DMA size larger then 64KB, size = 0x%x\n", size);
    return ret;
  }
  
  // Setup DMA
  if (data->flags & MMC_DATA_WRITE)
  {
    MsdcDmaEnable(0, (UINT8*)(data->src), size, TRUE);
  }
  else if (data->flags & MMC_DATA_READ)
  {
    MsdcDmaEnable(0, (UINT8*)(data->dest), size, FALSE);
  }

  return ret;
}

/*
 * Command handling
 */

static inline void msdc_get_short_reply(UINT32 ch, struct mtk_msdc_host *host,
struct mmc_cmd *cmd)
{
  cmd->response[0]  = MsdcReadResp(ch, 1);
}

static inline void msdc_get_long_reply(UINT32 ch, struct mtk_msdc_host *host,
struct mmc_cmd *cmd)
{
  cmd->response[0]  = MsdcReadResp(ch, 4);  // Bit 127~96
  cmd->response[1]  = MsdcReadResp(ch, 3);  // Bit 95~64
  cmd->response[2]  = MsdcReadResp(ch, 2);  // Bit 63~32
  cmd->response[3]  = MsdcReadResp(ch, 1);  // Bit 31~0
//  MSDC_LOG(MSDC_LOG_ERROR, "response[0] = 0x%x\n", cmd->response[0]);
//  MSDC_LOG(MSDC_LOG_ERROR, "response[1] = 0x%x\n", cmd->response[1]);
//  MSDC_LOG(MSDC_LOG_ERROR, "response[2] = 0x%x\n", cmd->response[2]);
//  MSDC_LOG(MSDC_LOG_ERROR, "response[3] = 0x%x\n", cmd->response[3]);
}

static int msdc_err_2_mmc_err(struct mtk_msdc_host *host, int msdc_err)
{
  int err = S_OK;

  switch(msdc_err)
  {
    case S_OK:
	err = S_OK;
	break;
  
    case E_CMD_TIMEOUT:
    case E_DATA_TIMEOUT:
      err = TIMEOUT;
	break;
		
    case E_NO_CARD:
      err = NO_CARD_ERR;
	break;	
		
    case E_SD_R3_OCR_BUSY:
    case E_INVALID_CARD:
    case E_INVALID_BLOCK_SIZE:
    case E_INVALID_ARG:
    case E_SD_APPCMD_FAILED:
    case E_RESPBUSY:
    case E_SD_NOT_SUPPORT_WIDE_BUS:
    case E_UNKNOWN_CARD:
    case E_CARD_IS_LOCKED:
    case E_FAIL:
      err = TIMEOUT;
	break;	
	
    case E_NO_RESPONSE:
    case E_DATA_CRC_ERROR:
    case E_CMD_RSP_CRC_ERR:
    case E_STATUS:
      err = UNUSABLE_ERR;
      break;
  }
  
  host->u4CmdError = err;
  	
  return err;
}

static int MsdcWaitDataLineIdle (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (0 != (MSDC_READ32(MSDC_PS(ch)) & 0x00010000))
    {
       break;
    }
    else if (u4DiffTime >= MSDC_WAIT_DATA_TIMEOUT_LIMIT)
    {
      MSDC_LOG(MSDC_LOG_ERROR, "timeout error !\n");
      return E_ERASE_TIMEOUT;
    }

    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}


/*
 * Sends a command out on the bus.  Takes the mmc pointer,
 * a command pointer, and an optional data pointer.
 */
static int msdc_send_command(struct mmc *mmc, struct mmc_cmd *cmd, struct mmc_data *data)
{
  INT32 i4Ret = 0;
  UINT32 ch = 0;
//  struct mmc_data *data = cmd->data;
  unsigned int sdcCmd = 0;	
  struct mtk_msdc_host *host = mmc->priv;	 

  ASSERT(host != NULL);
  ASSERT(cmd != NULL);

  host ->u4CmdError = 0;
  
  MSDC_LOG(MSDC_LOG_SDHWCMD, "Start - Sending cmd (%d)\n", cmd->cmdidx);


#ifdef CC_CARD_DETECT_PIN
  // handle interrupt status only in card inserted state.
  if (!(msdc_card_exist()))
  {
    MSDC_LOG(MSDC_LOG_ERROR, " No Card error.\n");
    return E_NO_CARD;
  }
#endif

  // Clear command state flag
  host->sdCmdState = MSDC_SD_CMD_INIT;
  
  // Check if rx/tx fifo is zero
  MsdcChkFifo(ch);

  // Clear interrupt Vector
  MsdcClrIntr(ch);    
    
  // Wait SDC bus idle
  if (MsdcWaitSDCBusIdle(ch) == FAILED)
  {
      MSDC_LOG(MSDC_LOG_ERROR, "Wait SDC Bus idle Failed !!, cmd (%d)\n", cmd->cmdidx);
      i4Ret = E_FAIL;
      goto end;
  }  
    
  // Setup command 1 : opcode & response type
  switch (cmd->resp_type)
  {
    case MMC_RSP_NONE:
      sdcCmd |= SDC_CMD_RSPTYPE_NO;
      MSDC_LOG(MSDC_LOG_SDHWCMD, "Response None\n");
      break;
    case MMC_RSP_R1:
      sdcCmd |= SDC_CMD_RSPTYPE_R1;
      MSDC_LOG(MSDC_LOG_SDHWCMD, "Response R1\n");
      break;
    case MMC_RSP_R1b:
      sdcCmd |= SDC_CMD_RSPTYPE_R1B;
      MSDC_LOG(MSDC_LOG_SDHWCMD, "Response R1B\n");
      break;
    case MMC_RSP_R2:
      sdcCmd |= SDC_CMD_RSPTYPE_R2;
      MSDC_LOG(MSDC_LOG_SDHWCMD, "Response R2\n");
      break;
    case MMC_RSP_R3:
      sdcCmd |= SDC_CMD_RSPTYPE_R3;
      MSDC_LOG(MSDC_LOG_SDHWCMD, "Response R3\n");
      break;
    default:
   //   BUG_ON(1);
      break;
  }

  // Set SDC_CMD.CMD
  sdcCmd |= (cmd->cmdidx & 0x3F);

  // Setup command 2 : Read/Write flag
  if (data)
  {
	MSDC_LOG(MSDC_LOG_SDHWCMD, "RW LBA = 0x%x \n", cmd->cmdarg);
    // Set Blk Length
    sdcCmd |= ((data->blocksize) << SDC_CMD_LEN_SHIFT);

    // Set Blk number    
    MSDC_WRITE32(SDC_BLK_NUM(ch), data->blocks);
    MSDC_LOG(MSDC_LOG_SDHWCMD, "blocks=%d, blocksize=%d \n", data->blocks, data->blocksize);
    if (data->flags & MMC_DATA_WRITE)
    {	
      host->wreqcnt++;

      if (data->blocks > 1)
      {
        sdcCmd |= (SDC_CMD_WRITE|DTYPE_MULTI_BLK);
      }
      else
      {
        sdcCmd |= (SDC_CMD_WRITE|DTYPE_SINGLE_BLK);
      }
    }
    else if (data->flags & MMC_DATA_READ)
    {
      host->rreqcnt++;
    
      if (data->blocks > 1)
      {
        sdcCmd |= (SDC_CMD_READ|DTYPE_MULTI_BLK);
      }
      else
      {
        sdcCmd |= (SDC_CMD_READ|DTYPE_SINGLE_BLK);
      }
    }
  }

  // Setup command 3 : Add stop bit for stop command
  if (cmd->cmdidx== 12)
  {
    sdcCmd |= SDC_CMD_STOP;
  }

 // cmd->error = 0;
  
  // Isssue command
  i4Ret = MsdcSendCmd(ch, host, sdcCmd, cmd->cmdarg);

  /*
   *  Wait for the request to complete.
   */
  // wait command ready.
  i4Ret = MsdcWaitCmdRdyOrTo(ch, host, cmd);
 
  if (i4Ret == S_OK)
  {
    /*
     * Do we expect a reply?
     */
    if (cmd->resp_type & MMC_RSP_PRESENT)
    {
      if (cmd->resp_type & MMC_RSP_136)
        msdc_get_long_reply(ch, host, cmd);
      else
        msdc_get_short_reply(ch, host, cmd);
    }
      
    /*
     * Prepare data before command send
     */
    if (data)
    {
      i4Ret = msdc_prepare_data(host, data);
      if (i4Ret)
      {
        MSDC_LOG(MSDC_LOG_ERROR, "msdc_prepare_data failed !!\n");
        goto end;
      }     

      i4Ret = MsdcWaitDataRdyOrTo(ch, host, data);
      if (i4Ret)
      {
        MsdcErrorHandling(ch, data);
        MSDC_LOG(MSDC_LOG_ERROR, "Wait data failed !\n");
        goto end;
      }
	  
	MsdcDmaDisable(ch);
      MsdcDmaStatusChk(ch);
  
    }  
    else
    {          
      if(cmd->cmdidx == MMC_CMD_ERASE)
      {
         i4Ret = MsdcWaitDataLineIdle(ch);
	}
	
      // Set command state flag
      host->sdCmdState = MSDC_SD_CMD_END;
    }
  }

end:
  msdc_err_2_mmc_err(host,i4Ret);
  MSDC_LOG(MSDC_LOG_SDHWCMD, "Finish - Sent cmd (%d), error: %d, i4Ret: 0x%x\n \n", cmd->cmdidx, host->u4CmdError, i4Ret);

  return host->u4CmdError;
}

/*****************************************************************************\
 *                                                                           *
 * MMC callbacks                                                             *
 *                                                                           *
\*****************************************************************************/




/*------------------------------------------------------------------------------
    MSDC host driver init/exit entry
------------------------------------------------------------------------------*/
static int msdc_init(struct mmc *mmc)
{
    struct mtk_msdc_host *host = &msdc_host;
    int ch = 0;

    // Reset fci_host
    memset(host, 0, sizeof(struct mtk_msdc_host));
    
    // set default bus_width
    mmc->bus_width = 1;

    // set initial clock (initial clock will be set in FCI_Init)
    mmc->clock = 1;
    
    // Initialize fci hardware
    return MsdcInit(ch);
}

static int msdc_initialize(bd_t *bis)
{
    struct mmc *mmc;
    struct mtk_msdc_host *host = &msdc_host;

    // Reset fci_host
    memset(host, 0, sizeof(struct mtk_msdc_host));
    
    mmc = malloc(sizeof(struct mmc));

    memset(mmc, 0, sizeof(struct mmc));

    sprintf(mmc->name, DRIVER_NAME);
    
    host->chip_id = CONFIG_CHIP_VER_CURR;
    host->base = MSDC_BASE_ADDR;
      
    mmc->priv = host;
    mmc->send_cmd = msdc_send_command;
    mmc->set_ios = msdc_set_ios;
    mmc->init = msdc_init;

    mmc->voltages |= (MMC_VDD_30_31 | MMC_VDD_31_32 | MMC_VDD_32_33 | 
                               MMC_VDD_33_34 | MMC_VDD_34_35 | MMC_VDD_35_36);

    mmc->host_caps = MMC_MODE_4BIT | MMC_MODE_HS | MMC_MODE_HS_52MHz;

    mmc->f_min = LOWEST_BUS_FREQ;
    mmc->f_max = BUS_CLK_FREQ;    

    mmc_register(mmc);

    printf("[%s] %d", __func__, (int)host->chip_id);
    printf(" At 0x%x config %d pwr %d\n", (int)host->base, _u4SDMultiFuncCfg, _u4SDPwrCdPinCfg);

    return 0;
}

int msdc_mmc_init(bd_t *bis)
{
    return msdc_initialize(bis);
}


