/*
 * Copyright 2008, Freescale Semiconductor, Inc
 * Andy Fleming
 *
 * Based vaguely on the Linux code
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <command.h>

#include <part.h>
#include <malloc.h>
#include <ubi_uboot.h>

#include <linux/list.h>
#include <div64.h>

#include "emmc.h"
#include "emmc_msdc.h"


#ifndef	TRUE
#define TRUE            1
#endif
#ifndef FALSE
#define FALSE           0
#endif



struct emmc *emmc_ptr = NULL;



static int emmc_send_cmd(struct emmc *emmc, struct emmc_cmd *cmd, struct emmc_data *data)
{
	return emmc->send_cmd(emmc, cmd, data);
}

static int emmc_set_blocklen(struct emmc *emmc, int len)
{
	struct emmc_cmd cmd;

	cmd.cmdidx = MMC_CMD_SET_BLOCKLEN;
	cmd.resp_type = MMC_RSP_R1;
	cmd.cmdarg = len;
	cmd.flags = 0;

	return emmc_send_cmd(emmc, &cmd, NULL);
}

static struct emmc *find_emmc_device(int dev_num)
{
	//printf("MMC Device %d not found\n", dev_num);

	return emmc_ptr;
}

#if 0
static uint emmc_bwrite(uint start, uint blkcnt, const void*src)
{
	struct emmc_cmd cmd;
	struct emmc_data data;
	int err;
	int stoperr = 0;
	struct emmc *emmc = find_emmc_device(0);
	int blklen;

	if (!emmc)
		return -1;
       printf("[emmc_bwrite]: start = 0x%x, blkcnt = %d \n", start, blkcnt);
	blklen = emmc->write_bl_len;

	err = emmc_set_blocklen(emmc, emmc->write_bl_len);

	if (err) {
		printf("set write bl len failed\n\r");
		return err;
	}

	if (blkcnt > 1)
		cmd.cmdidx = MMC_CMD_WRITE_MULTIPLE_BLOCK;
	else
		cmd.cmdidx = MMC_CMD_WRITE_SINGLE_BLOCK;

	if (emmc->high_capacity)
		cmd.cmdarg = start;
	else
		cmd.cmdarg = start * blklen;

	cmd.resp_type = MMC_RSP_R1;
	cmd.flags = 0;

	data.src = src;
	data.blocks = blkcnt;
	data.blocksize = blklen;
	data.flags = MMC_DATA_WRITE;

	err = emmc_send_cmd(emmc, &cmd, &data);

	if (err) {
		printf("emmc write failed\n\r");
		return err;
	}

	if (blkcnt > 1) {
		cmd.cmdidx = MMC_CMD_STOP_TRANSMISSION;
		cmd.cmdarg = 0;
		cmd.resp_type = MMC_RSP_R1b;
		cmd.flags = 0;
		stoperr = emmc_send_cmd(emmc, &cmd, NULL);
	}

	return blkcnt;
}
#endif

static uint  emmc_bwrite(uint start, uint blkcnt, const void*src)
{
	struct emmc_cmd cmd;
	struct emmc_data data;
	int err;
	int stoperr = 0;
	struct emmc *emmc = find_emmc_device(0);
	int blklen;
      int i, curcount;
      uint startLBA;
	  
	if (!emmc)
		return 0;

	printf("[emmc_bwrite]: startLBA=0x%x, blkcnt=%d \n", start, blkcnt);
	blklen = emmc->read_bl_len;

	err = emmc_set_blocklen(emmc, emmc->read_bl_len);

	if (err) {
		printf("set read bl len failed\n\r");
		return 0;
	}
	for (i = start; i < start + blkcnt; i+=MSDC_MAX_BLOCK_NUM, src += (emmc->read_bl_len*curcount))
	{
	      // 1. get current WRITE block count
		if((start + blkcnt - i) > MSDC_MAX_BLOCK_NUM)
			curcount = MSDC_MAX_BLOCK_NUM;
		else
			curcount = start + blkcnt - i;
          //   printf(" 2: curcount=0x%x \n", curcount);
			 
		// 2. get current WRITE LAB;	
		startLBA = i;
		
		if (curcount > 1)
			cmd.cmdidx = MMC_CMD_WRITE_MULTIPLE_BLOCK;
		else
			cmd.cmdidx = MMC_CMD_WRITE_SINGLE_BLOCK;

		if (emmc->high_capacity)
			cmd.cmdarg = startLBA;
		else
			cmd.cmdarg = startLBA * blklen;

		cmd.resp_type = MMC_RSP_R1;
		cmd.flags = 0;

		data.src = src;
		data.blocks = curcount;
		data.blocksize = blklen;
		data.flags = MMC_DATA_WRITE;
             // 3. send WRITE command;
		err = emmc_send_cmd(emmc, &cmd, &data);

		if (err) {
			printf("emmc write failed\n\r");
			return 0;
		}
         
		 // 4. send STOP command;
		if (blkcnt > 1) {
			cmd.cmdidx = MMC_CMD_STOP_TRANSMISSION;
			cmd.cmdarg = 0;
			cmd.resp_type = MMC_RSP_R1b;
			cmd.flags = 0;
			stoperr = emmc_send_cmd(emmc, &cmd, NULL);
			
			if(stoperr != 0){
				return 0;
			}
		}

	}

	return blkcnt;
}

static uint  emmc_bread(uint start, uint blkcnt, void*dst)
{
	struct emmc_cmd cmd;
	struct emmc_data data;
	int err;
	int stoperr = 0;
	struct emmc *emmc = find_emmc_device(0);
	int blklen;
      int i, curcount;
      uint startLBA;
	  
	if (!emmc)
		return 0;

	printf("[emmc_bread]: startLBA=0x%x, blkcnt=%d \n", start, blkcnt);
	blklen = emmc->read_bl_len;

	err = emmc_set_blocklen(emmc, emmc->read_bl_len);

	if (err) {
		printf("set read bl len failed\n\r");
		return 0;
	}
	for (i = start; i < start + blkcnt; i+=MSDC_MAX_BLOCK_NUM, dst += (emmc->read_bl_len*curcount))
	{
	      // 1. get current READ block count
		if((start + blkcnt - i) > MSDC_MAX_BLOCK_NUM)
			curcount = MSDC_MAX_BLOCK_NUM;
		else
			curcount = start + blkcnt - i;
            // printf(" 2: curcount=0x%x \n", curcount);
			 
		// 2. get current READ LAB;	
		startLBA = i;
		
		if (curcount > 1)
			cmd.cmdidx = MMC_CMD_READ_MULTIPLE_BLOCK;
		else
			cmd.cmdidx = MMC_CMD_READ_SINGLE_BLOCK;

		if (emmc->high_capacity)
			cmd.cmdarg = startLBA;
		else
			cmd.cmdarg = startLBA * blklen;

		cmd.resp_type = MMC_RSP_R1;
		cmd.flags = 0;

		data.dest = dst;
		data.blocks = curcount;
		data.blocksize = blklen;
		data.flags = MMC_DATA_READ;
             // 3. send READ command;
		err = emmc_send_cmd(emmc, &cmd, &data);

		if (err) {
			printf("emmc read failed\n\r");
			return 0;
		}
         
		 // 4. send STOP command;
		if (blkcnt > 1) {
			cmd.cmdidx = MMC_CMD_STOP_TRANSMISSION;
			cmd.cmdarg = 0;
			cmd.resp_type = MMC_RSP_R1b;
			cmd.flags = 0;
			stoperr = emmc_send_cmd(emmc, &cmd, NULL);
						
			if(stoperr != 0){
				return 0;
			}
		}
	}

	return blkcnt;
}

static int emmc_write_length_unalign(uint start, uint length, void* src) 
{
	uint startAddr, blkCnt;
	int err = EMMC_OK;
	uint firstLen, secondLen;
	uint firstAddr, secondAddr;
	uint unalignSize = length%EMMC_BLOCK_LEN;
	u8 *srcBuf, *tempBuf;
	
	printf("[%s] enter: \n", __func__);
	
	tempBuf = kzalloc(EMMC_BLOCK_LEN, GFP_KERNEL);
	ASSERT(tempBuf != NULL);
	ASSERT(!(start % EMMC_BLOCK_LEN));
	
	firstLen = length - unalignSize;
	secondLen = unalignSize;
	firstAddr = start;
	secondAddr = start + firstLen;
	printf("[%s] firstLen=0x%x, secondLen=0x%x,\n", __func__, firstLen, secondLen);
	printf("[%s] firstAddr=0x%x, secondAddr=0x%x,\n", __func__, firstAddr, secondAddr);

	// 1. First partiton operate
	if(length >= EMMC_BLOCK_LEN){
		startAddr = firstAddr>>EMMC_BLK_LEN_SHIFT;
		blkCnt = firstLen>>EMMC_BLK_LEN_SHIFT;
		if(blkCnt != emmc_bwrite(startAddr, blkCnt, src)){
			printf("[%s] Write fail 1 \n", __func__);
			err = EMMC_FAIL;
		}
	}

	// 2. Second partiton operate
	if(secondLen > 0){
		startAddr = secondAddr>>EMMC_BLK_LEN_SHIFT;
		blkCnt = 1;
		if(blkCnt != emmc_bread(startAddr, blkCnt, tempBuf)){
			printf("[%s] read fail 2 \n", __func__);
			err = EMMC_FAIL;
		}
		
		srcBuf = (u8*)src + firstLen;
		memcpy(tempBuf, srcBuf, secondLen);
		if(blkCnt != emmc_bwrite(startAddr, blkCnt, tempBuf)){
			printf("[%s] Write fail 3 \n", __func__);
			err = EMMC_FAIL;
		}
	}
	
	// 3. release resource	
	kfree(tempBuf);
	
	return err;
}

static int emmc_write_address_unalign(uint start, uint length, void* src) 
{
	uint startAddr, blkCnt;
	int err = EMMC_OK;
	uint firstLen, secondLen;
	uint firstAddr, secondAddr;
	uint unalignSize = start%EMMC_BLOCK_LEN;
	u8 *srcBuf, *tempBuf;
	
	printf("[%s] enter: \n", __func__);
	
	tempBuf = kzalloc(EMMC_BLOCK_LEN, GFP_KERNEL);
	ASSERT(tempBuf != NULL);
	ASSERT(start % EMMC_BLOCK_LEN);

	firstAddr = start - unalignSize;	
	secondAddr = firstAddr + EMMC_BLOCK_LEN;
	firstLen = EMMC_BLOCK_LEN - unalignSize;
	if(length <= firstLen){
		firstLen = length;
		secondLen = 0;
	}else{
		secondLen = length - firstLen;
	}
	printf("[%s] firstLen=0x%x, secondLen=0x%x,\n", __func__, firstLen, secondLen);
	printf("[%s] firstAddr=0x%x, secondAddr=0x%x,\n", __func__, firstAddr, secondAddr);

	// 1. First partiton operate
	startAddr = firstAddr>>EMMC_BLK_LEN_SHIFT;
	blkCnt = 1;	
	if(blkCnt != emmc_bread(startAddr, blkCnt, tempBuf)){
		printf("[%s] Read fail 1\n", __func__);
		err = EMMC_FAIL;
	}
	
	srcBuf = (u8*)src;
	memcpy(tempBuf+unalignSize, srcBuf, firstLen);
	if(blkCnt != emmc_bwrite(startAddr, blkCnt, tempBuf)){
		printf("[%s] Write fail 2\n", __func__);
		err = EMMC_FAIL;
	}

	// 2. Second partiton operate
	if(secondLen > 0){
		srcBuf +=firstLen;
		err = emmc_write_length_unalign(secondAddr, secondLen, srcBuf);
	}
	
	return err;

}


/*-----------------------------------------------------------------------------
 * Name: emmc_write
 * Description: Write data from eMMC card
 *
 * Inputs:    	start		Dest address to write(Byte)
 *			length			total size(Byte) to write
 				src			data buffer pointer
 * Outputs: -

 * Returns: 	EMMC_OK		emmc_read success
 *			EMMC_FAIL		emmc_read failed
 ----------------------------------------------------------------------------*/
int emmc_write(uint start, uint length, void* src) 
{
	uint startAddr, blkCnt;
	int err = EMMC_OK;
	BOOL fgStartAddrAlign =TRUE;
	BOOL fgLengthAlign =TRUE;
	BOOL fgBufferAlign =TRUE;
	struct emmc *emmc = find_emmc_device(0);
	u64 cap; // device total capacity
	
	printf("[%s] start=0x%x, length=0x%x, src=0x%x\n", __func__,start,length,(UINT32)src);

	cap = emmc->capacity<<20;
	if((start + length) > cap){
		printf("[%s] The written content is exceed the device capacity\n", __func__);
		return EMMC_FAIL;
	}

	if(start%EMMC_BLOCK_LEN)
		fgStartAddrAlign = FALSE;
	if(length%EMMC_BLOCK_LEN)
		fgLengthAlign = FALSE;
	if(((uint)src) % (sizeof(uint)))
		fgBufferAlign = FALSE;
	
	printf("[%s] fgStartAddrAlign/fgLengthAlign/fgBufferAlign=%d:%d:%d \n", __func__,fgStartAddrAlign,fgLengthAlign,fgBufferAlign);

	if(!fgBufferAlign){
		printf("[%s] dst pointer is not align\n", __func__);
		return EMMC_FAIL;
	}

	if(fgStartAddrAlign & fgLengthAlign){
		// addr=y, length=y
		startAddr = start>>EMMC_BLK_LEN_SHIFT;
		blkCnt = length>>EMMC_BLK_LEN_SHIFT;
		if(blkCnt != emmc_bwrite(startAddr, blkCnt, src)){
			printf("[%s] Write fail 1 \n", __func__);
			err = EMMC_FAIL;
		}
	}
	else if(fgStartAddrAlign & !fgLengthAlign){
		// addr=y, length=n
		err = emmc_write_length_unalign(start, length, src);
	}
	else if(!fgStartAddrAlign){
		// addr=n, length=y/n
		err = emmc_write_address_unalign(start, length, src);
	}
	
	return err;
}


static int emmc_read_length_unalign(uint start, uint length, void* dst) 
{
	uint startAddr, blkCnt;
	int err = EMMC_OK;
	uint firstLen, secondLen;
	uint firstAddr, secondAddr;
	uint unalignSize = length%EMMC_BLOCK_LEN;
	u8 *dstBuf, *tempBuf;
	
	//printf("[%s] enter: \n", __func__);
	
	tempBuf = kzalloc(EMMC_BLOCK_LEN, GFP_KERNEL);
	ASSERT(tempBuf != NULL);
	
	firstLen = length - unalignSize;
	secondLen = unalignSize;
	firstAddr = start;
	secondAddr = start + firstLen;
	printf("[%s] firstLen=0x%x, secondLen=0x%x,\n", __func__, firstLen, secondLen);
	printf("[%s] firstAddr=0x%x, secondAddr=0x%x,\n", __func__, firstAddr, secondAddr);

	// 1. First partiton operate
	if(length >= EMMC_BLOCK_LEN){
		startAddr = firstAddr>>EMMC_BLK_LEN_SHIFT;
		blkCnt = firstLen>>EMMC_BLK_LEN_SHIFT;
		if(blkCnt != emmc_bread(startAddr, blkCnt, dst)){
			printf("[%s] Read fail 1 \n", __func__);
			err = EMMC_FAIL;
		}
	}

	// 2. Second partiton operate
	if(secondLen > 0){
		startAddr = secondAddr>>EMMC_BLK_LEN_SHIFT;
		blkCnt = 1;		
		if(blkCnt != emmc_bread(startAddr, blkCnt, tempBuf)){
			printf("[%s] Read fail 2 \n", __func__);
			err = EMMC_FAIL;
		}
		dstBuf = (u8*)dst + firstLen;
		memcpy(dstBuf, tempBuf, secondLen);
	}
	// 3. release resource	
	kfree(tempBuf);
	
	return err;
}

static int emmc_read_address_unalign(uint start, uint length, void* dst) 
{
	uint startAddr, blkCnt;
	int err = EMMC_OK;
	uint firstLen, secondLen;
	uint firstAddr, secondAddr;
	uint unalignSize = start%EMMC_BLOCK_LEN;
	u8 *dstBuf, *tempBuf;
	
	//printf("[%s] enter: \n", __func__);
	
	tempBuf = kzalloc(EMMC_BLOCK_LEN, GFP_KERNEL);
	ASSERT(tempBuf != NULL);
	

	firstAddr = start - unalignSize;	
	secondAddr = firstAddr + EMMC_BLOCK_LEN;
	firstLen = EMMC_BLOCK_LEN - unalignSize;
	if(length <= firstLen){
		firstLen = length;
		secondLen = 0;
	}else{
		secondLen = length - firstLen;
	}
	printf("[%s] firstLen=0x%x, secondLen=0x%x,\n", __func__, firstLen, secondLen);
	printf("[%s] firstAddr=0x%x, secondAddr=0x%x,\n", __func__, firstAddr, secondAddr);

	// 1. First partiton operate
	startAddr = firstAddr>>EMMC_BLK_LEN_SHIFT;
	blkCnt = 1;
	
	if(blkCnt != emmc_bread(startAddr, blkCnt, tempBuf)){
		printf("[%s] Read fail \n", __func__);
		err = EMMC_FAIL;
	}
	dstBuf = (u8*)dst;
	memcpy(dstBuf, tempBuf+unalignSize, firstLen);
	
	// 2. Second partiton operate
	if(secondLen > 0){
		dstBuf +=firstLen;
		err = emmc_read_length_unalign(secondAddr, secondLen, dstBuf);
	}
	
	return err;

}

/*-----------------------------------------------------------------------------
 * Name: emmc_read
 * Description: Read data from eMMC card
 *
 * Inputs:    	start		Dest address to read(Byte)
 *			length		Total size(Byte) to read
 			dst			data buffer pointer
 * Outputs: -

 * Returns: 	EMMC_OK		emmc_read success
 *			EMMC_FAIL		emmc_read failed
 ----------------------------------------------------------------------------*/
int emmc_read(uint start, uint length, void* dst) 
{
	uint startAddr, blkCnt;
	int err = EMMC_OK;
	BOOL fgStartAddrAlign =TRUE;
	BOOL fgLengthAlign =TRUE;
	BOOL fgBufferAlign =TRUE;
	struct emmc *emmc = find_emmc_device(0);
	u64 cap; // device total capacity
	
	printf("[%s] start=0x%x, length=0x%x, dst=0x%x\n", __func__,start,length,(UINT32)dst);

	cap = emmc->capacity<<20;
	if((start + length) > cap){
		printf("[%s] The read content is exceed the device capacity\n", __func__);
		return EMMC_FAIL;
	}

	if(start%EMMC_BLOCK_LEN)
		fgStartAddrAlign = FALSE;
	if(length%EMMC_BLOCK_LEN)
		fgLengthAlign = FALSE;
	if(((uint)dst) % (sizeof(uint)))
		fgBufferAlign = FALSE;
	
	printf("[%s] fgStartAddrAlign/fgLengthAlign/fgBufferAlign=%d:%d:%d \n", __func__,fgStartAddrAlign,fgLengthAlign,fgBufferAlign);

	if(!fgBufferAlign){
		printf("[%s] dst pointer is not align\n", __func__);
		return EMMC_FAIL;
	}

	if(fgStartAddrAlign & fgLengthAlign){
		// addr=y, length=y
		startAddr = start>>EMMC_BLK_LEN_SHIFT;
		blkCnt = length>>EMMC_BLK_LEN_SHIFT;
		if(blkCnt != emmc_bread(startAddr, blkCnt, dst)){
			printf("[%s] Read fail 1 \n", __func__);
			err = EMMC_FAIL;
		}
	}
	else if(fgStartAddrAlign & !fgLengthAlign){
		// addr=y, length=n
		err = emmc_read_length_unalign(start, length, dst);
	}
	else if(!fgStartAddrAlign){
		// addr=n, length=y/n
		err = emmc_read_address_unalign(start, length, dst);
	}
	
	return err;
}


static int emmc_go_idle(struct emmc* emmc)
{
	struct emmc_cmd cmd;
	int err;

	udelay(1000);

	cmd.cmdidx = MMC_CMD_GO_IDLE_STATE;
	cmd.cmdarg = 0;
	cmd.resp_type = MMC_RSP_NONE;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
		return err;

	udelay(2000);

	return 0;
}

/*

// start: start address of block unit
int emmc_erase_blk_start_addr(struct emmc* emmc, unsigned long start)
{
	struct emmc_cmd cmd;
	int err;

	
		
	cmd.cmdidx = MMC_CMD_ERASE_START_ADDR;
	if(emmc->high_capacity)
		cmd.cmdarg = start;
	else
		cmd.cmdarg = start<<9;
	
	cmd.resp_type = MMC_RSP_R1;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
		return err;

	return 0;
}


int emmc_erase_blk_end_addr(struct emmc* emmc, unsigned long end)
{
	struct emmc_cmd cmd;
	int err;

	cmd.cmdidx = MMC_CMD_ERASE_END_ADDR;
	if(emmc->high_capacity)
		cmd.cmdarg = end;
	else
		cmd.cmdarg = end<<9;

	cmd.resp_type = MMC_RSP_R1;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
		return err;

	return 0;
}

int emmc_erase_blk(struct emmc* emmc)
{
	struct emmc_cmd cmd;
	int err;

	cmd.cmdidx = MMC_CMD_ERASE;
	cmd.cmdarg = 0;
	cmd.resp_type = MMC_RSP_R1b;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
		return err;

	return 0;
}

int emmc_erase_block(struct emmc* emmc, unsigned long start, unsigned long blklen)
{
	int i, err;
      unsigned long total_blk = (unsigned long)((emmc->capacity)>>9);
      unsigned long max_erase_num = (emmc->erase_grp_size << 4);
      unsigned long end_addr = 0;	  
	  
      printf("[%s] total_blk = 0x%x \n", __func__, total_blk);
	  printf("[%s] Org blklen = 0x%x \n", __func__, blklen);
	if((start+blklen) > total_blk)
		blklen = total_blk - start;
      printf("[%s] blklen = 0x%x \n", __func__, blklen);
	for(i = 0; i < blklen; i++)
	{	      
		if(blklen > ((i+1)*max_erase_num))
			end_addr = start + ((i+1)*max_erase_num);
		else
			end_addr = start + blklen;
             printf("[%s] [%d] end_addr = 0x%x \n", __func__, i, end_addr);
		err = emmc_erase_blk_start_addr(emmc,  (start+i*max_erase_num));
		if(err != 0)
		{
			printf("[%s] emmc_erase_blk_start_addr: err \n", __func__);
			goto erase_quit;
		}
		err = emmc_erase_blk_end_addr(emmc,  end_addr);
		if(err != 0)
		{
			printf("[%s] emmc_erase_blk_end_addr: err \n", __func__);
			goto erase_quit;
		}

		err = emmc_erase_blk(emmc);
		if(err != 0)
		{
			printf("[%s] emmc_erase_blk: err \n", __func__);
			goto erase_quit;
		}

		if(end_addr >= (start + blklen))
			break;

	}
	
	return 0;
erase_quit:
	printf("[%s] err = 0x%x \n", __func__, err);
	return err;
}
*/

static int emmc_sd_send_op_cond(struct emmc *emmc)
{
	int timeout = 1000;
	int err;
	struct emmc_cmd cmd;

	do {
		cmd.cmdidx = MMC_CMD_APP_CMD;
		cmd.resp_type = MMC_RSP_R1;
		cmd.cmdarg = 0;
		cmd.flags = 0;

		err = emmc_send_cmd(emmc, &cmd, NULL);

		if (err)
			return err;

		cmd.cmdidx = SD_CMD_APP_SEND_OP_COND;
		cmd.resp_type = MMC_RSP_R3;
		cmd.cmdarg = emmc->voltages;

		if (emmc->version == SD_VERSION_2)
			cmd.cmdarg |= OCR_HCS;
              
		err = emmc_send_cmd(emmc, &cmd, NULL);

		if (err)
			return err;

		udelay(10000);
	} while ((!(cmd.response[0] & OCR_BUSY)) && timeout--);

	if (timeout <= 0)
		return EMMC_UNUSABLE_ERR;

	if (emmc->version != SD_VERSION_2)
		emmc->version = SD_VERSION_1_0;

	emmc->ocr = cmd.response[0];

	emmc->high_capacity = ((emmc->ocr & OCR_HCS) == OCR_HCS);
	emmc->rca = 0;

	return 0;
}

static int emmc_send_op_cond(struct emmc *emmc)
{
	int timeout = 1000;
	struct emmc_cmd cmd;
	int err;

	/* Some cards seem to need this */
	emmc_go_idle(emmc);

	do {
		cmd.cmdidx = MMC_CMD_SEND_OP_COND;
		cmd.resp_type = MMC_RSP_R3;
		cmd.cmdarg = OCR_HCS | emmc->voltages;
		cmd.flags = 0;

		err = emmc_send_cmd(emmc, &cmd, NULL);
		
             printf("[%s] response[0] = 0x%x \n", __func__, cmd.response[0]);
			 
		if (err)
			return err;

		udelay(100000);
	} while (!(cmd.response[0] & OCR_BUSY) && timeout--);

	if (timeout <= 0)
		return EMMC_UNUSABLE_ERR;

	emmc->version = MMC_VERSION_UNKNOWN;
	emmc->ocr = cmd.response[0];

	emmc->high_capacity = ((emmc->ocr & OCR_HCS) == OCR_HCS);
	emmc->rca = 0;

	return 0;
}


static int emmc_send_ext_csd(struct emmc *emmc, char *ext_csd)
{
	struct emmc_cmd cmd;
	struct emmc_data data;
	int err;

	/* Get the Card Status Register */
	cmd.cmdidx = MMC_CMD_SEND_EXT_CSD;
	cmd.resp_type = MMC_RSP_R1;
	cmd.cmdarg = 0;
	cmd.flags = 0;

	data.dest = ext_csd;
	data.blocks = 1;
	data.blocksize = 512;
	data.flags = MMC_DATA_READ;

	err = emmc_send_cmd(emmc, &cmd, &data);

	return err;
}


static int emmc_switch(struct emmc *emmc, u8 set, u8 index, u8 value)
{
	struct emmc_cmd cmd;

	cmd.cmdidx = MMC_CMD_SWITCH;
	cmd.resp_type = MMC_RSP_R1b;
	cmd.cmdarg = (MMC_SWITCH_MODE_WRITE_BYTE << 24) |
		(index << 16) |
		(value << 8);
	cmd.flags = 0;

	return emmc_send_cmd(emmc, &cmd, NULL);
}

static int emmc_change_freq(struct emmc *emmc)
{
	char ext_csd[512];
	char cardtype;
	int err;

	emmc->card_caps = 0;

	/* Only version 4 supports high-speed */
	if (emmc->version < MMC_VERSION_4){
		printf("[%s]emmc->version < MMC_VERSION_4 \n", __func__);
		return 0;
	}
	emmc->card_caps |= MMC_MODE_4BIT;

#if 1 // Don't switch high speed

	err = emmc_send_ext_csd(emmc, ext_csd);
      printf("[%s]BOOT SIZE: ext_csd[226]=0x%x \n", __func__, ext_csd[226]);
      printf("[%s]BOOT CFG: ext_csd[179]=0x%x \n", __func__, ext_csd[179]);
      printf("[%s]csd_struct: ext_csd[192]=0x%x \n", __func__, ext_csd[192]);

   //   printf("[%s] ext_csd[175/177/179/181]=0x%x,0x%x,0x%x,0x%x \n", __func__, ext_csd[175],ext_csd[177], ext_csd[179],ext_csd[181]);
    //  printf("[%s] ext_csd[183/185/187/189]=0x%x,0x%x,0x%x,0x%x \n", __func__, ext_csd[183],ext_csd[185], ext_csd[187],ext_csd[189]);
   //   printf("[%s] ext_csd[191/192/194/196]=0x%x,0x%x,0x%x,0x%x \n", __func__, ext_csd[191],ext_csd[192], ext_csd[194],ext_csd[196]);
   //   printf("[%s] ext_csd[228/226/225/224]=0x%x,0x%x,0x%x,0x%x \n", __func__, ext_csd[228],ext_csd[226], ext_csd[225],ext_csd[224]);
	
       printf("[%s] ext_csd[212~215]=0x%x,0x%x,0x%x,0x%x \n", __func__, ext_csd[212],ext_csd[213], ext_csd[214],ext_csd[215]);
	if (err)
		return err;

	if (ext_csd[212] || ext_csd[213] || ext_csd[214] || ext_csd[215]){
		uint cap = (ext_csd[212] + (ext_csd[213]<<8) + (ext_csd[214]<<16) + (ext_csd[215]<<24));
		emmc->high_capacity = 1;
		emmc->capacity = cap/((1024*1024)/emmc->read_bl_len);
		printf("[%s] emmc->capacity = %d MB \n", __func__, emmc->capacity);
	}

	// set  erase_grp_size
	if(ext_csd[224])
		emmc->erase_grp_size = 1024*ext_csd[224];  // 1024blk * 512k num
	else
		emmc->erase_grp_size = 1024;

	cardtype = ext_csd[196] & 0xf;
      printf("[%s] ext_csd[196]=0x%x \n", __func__, ext_csd[196]);
	err = emmc_switch(emmc, EXT_CSD_CMD_SET_NORMAL, EXT_CSD_HS_TIMING, 1);

	if (err)
		return err;

	/* Now check to see that it worked */
	err = emmc_send_ext_csd(emmc, ext_csd);

	if (err)
		return err;
       printf("[%s] ext_csd[185]=0x%x \n", __func__, ext_csd[185]);
	/* No high-speed support */
	if (!ext_csd[185])
		return 0;

	/* High Speed is set, there are two types: 52MHz and 26MHz */
	if (cardtype & MMC_HS_52MHZ)
		emmc->card_caps |= MMC_MODE_HS_52MHz | MMC_MODE_HS;
	else
		emmc->card_caps |= MMC_MODE_HS;
#endif

	return 0;
}

static int emmc_sd_switch(struct emmc *emmc, int mode, int group, u8 value, u8 *resp)
{
	struct emmc_cmd cmd;
	struct emmc_data data;

	/* Switch the frequency */
	cmd.cmdidx = SD_CMD_SWITCH_FUNC;
	cmd.resp_type = MMC_RSP_R1;
	cmd.cmdarg = (mode << 31) | 0xffffff;
	cmd.cmdarg &= ~(0xf << (group * 4));
	cmd.cmdarg |= value << (group * 4);
	cmd.flags = 0;

	data.dest = (char *)resp;
	data.blocksize = 64;
	data.blocks = 1;
	data.flags = MMC_DATA_READ;

	return emmc_send_cmd(emmc, &cmd, &data);
}


static int emmc_sd_change_freq(struct emmc *emmc)
{
	int err;
	struct emmc_cmd cmd;
	uint scr[2];
	uint switch_status[16];
	struct emmc_data data;
	int timeout;

	emmc->card_caps = 0;

	/* Read the SCR to find out if this card supports higher speeds */
	cmd.cmdidx = MMC_CMD_APP_CMD;
	cmd.resp_type = MMC_RSP_R1;
	cmd.cmdarg = emmc->rca << 16;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
		return err;

	cmd.cmdidx = SD_CMD_APP_SEND_SCR;
	cmd.resp_type = MMC_RSP_R1;
	cmd.cmdarg = 0;
	cmd.flags = 0;

	timeout = 3;

retry_scr:
	data.dest = (char *)&scr;
	data.blocksize = 8;
	data.blocks = 1;
	data.flags = MMC_DATA_READ;

	err = emmc_send_cmd(emmc, &cmd, &data);

	if (err) {
		if (timeout--)
			goto retry_scr;

		return err;
	}

	emmc->scr[0] = __be32_to_cpu(scr[0]);
	emmc->scr[1] = __be32_to_cpu(scr[1]);

	switch ((emmc->scr[0] >> 24) & 0xf) {
		case 0:
			emmc->version = SD_VERSION_1_0;
			break;
		case 1:
			emmc->version = SD_VERSION_1_10;
			break;
		case 2:
			emmc->version = SD_VERSION_2;
			break;
		default:
			emmc->version = SD_VERSION_1_0;
			break;
	}

	if (emmc->scr[0] & SD_DATA_4BIT)
		emmc->card_caps |= MMC_MODE_4BIT;
	
	/* Version 1.0 doesn't support switching */
	if (emmc->version == SD_VERSION_1_0)
		return 0;

	timeout = 4;
	while (timeout--) {
		err = emmc_sd_switch(emmc, SD_SWITCH_CHECK, 0, 1,
				(u8 *)&switch_status);

		if (err)
			return err;

		/* The high-speed function is busy.  Try again */
		if (!(__be32_to_cpu(switch_status[7]) & SD_HIGHSPEED_BUSY))
			break;
	}

	/* If high-speed isn't supported, we return */
	if (!(__be32_to_cpu(switch_status[3]) & SD_HIGHSPEED_SUPPORTED))
		return 0;

	err = emmc_sd_switch(emmc, SD_SWITCH_SWITCH, 0, 1, (u8 *)&switch_status);

	if (err)
		return err;

	if ((__be32_to_cpu(switch_status[4]) & 0x0f000000) == 0x01000000)
		emmc->card_caps |= MMC_MODE_HS;

	return 0;
}

/* frequency bases */
/* divided by 10 to be nice to platforms without floating point */
static int emmc_fbase[] = {
	10000,
	100000,
	1000000,
	10000000,
};

/* Multiplier values for TRAN_SPEED.  Multiplied by 10 to be nice
 * to platforms without floating point.
 */
static int emmc_multipliers[] = {
	0,	/* reserved */
	10,
	12,
	13,
	15,
	20,
	25,
	30,
	35,
	40,
	45,
	50,
	55,
	60,
	70,
	80,
};

static void emmc_set_ios(struct emmc *emmc)
{
	emmc->set_ios(emmc);
}

static void emmc_set_clock(struct emmc *emmc, uint clock)
{
	if (clock > emmc->f_max)
		clock = emmc->f_max;

	if (clock < emmc->f_min)
		clock = emmc->f_min;

	emmc->clock = clock;

	emmc_set_ios(emmc);
}

static void emmc_set_bus_width(struct emmc *emmc, uint width)
{
	emmc->bus_width = width;

	emmc_set_ios(emmc);
}

static int emmc_startup(struct emmc *emmc)
{
	int err;
	uint mult, freq;
	u64 cmult, csize;
	struct emmc_cmd cmd;

	/* Put the Card in Identify Mode */
	cmd.cmdidx = MMC_CMD_ALL_SEND_CID;
	cmd.resp_type = MMC_RSP_R2;
	cmd.cmdarg = 0;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
	{
	   printf("[%s] SEND_CID error !!\n", __func__);
		return err;
   }

	memcpy(emmc->cid, cmd.response, 16);
      printf("[%s] cid[0] = 0x%x \n", __func__, emmc->cid[0]);
      printf("[%s] cid[1] = 0x%x \n", __func__, emmc->cid[1]);
	printf("[%s] cid[2] = 0x%x \n", __func__, emmc->cid[2]);
	printf("[%s] cid[3] = 0x%x \n", __func__, emmc->cid[3]);
	  
	/*
	 * For MMC cards, set the Relative Address.
	 * For SD cards, get the Relatvie Address.
	 * This also puts the cards into Standby State
	 */
	cmd.cmdidx = SD_CMD_SEND_RELATIVE_ADDR;
	cmd.cmdarg = emmc->rca << 16;
	cmd.resp_type = MMC_RSP_R6;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
	{
	   printf("[%s] SEND_RCA error !!\n", __func__);
		return err;
   }

	if (IS_SD(emmc))
		emmc->rca = (cmd.response[0] >> 16) & 0xffff;

	/* Get the Card-Specific Data */
	cmd.cmdidx = MMC_CMD_SEND_CSD;
	cmd.resp_type = MMC_RSP_R2;
	cmd.cmdarg = emmc->rca << 16;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
	{
	   printf("[%s] SEND_CSD error !!\n", __func__);
		return err;
   }
	emmc->csd[0] = cmd.response[0];
	emmc->csd[1] = cmd.response[1];
	emmc->csd[2] = cmd.response[2];
	emmc->csd[3] = cmd.response[3];
	
      printf("[%s] csd[0] = 0x%x \n", __func__, emmc->csd[0]);
      printf("[%s] csd[1] = 0x%x \n", __func__, emmc->csd[1]);
	printf("[%s] csd[2] = 0x%x \n", __func__, emmc->csd[2]);
	printf("[%s] csd[3] = 0x%x \n", __func__, emmc->csd[3]);
	  
	if (emmc->version == MMC_VERSION_UNKNOWN) {
		int version = (cmd.response[0] >> 26) & 0xf;
             printf("[%s] version = 0x%x \n", __func__, version);
		switch (version) {
			case 0:
				emmc->version = MMC_VERSION_1_2;
				break;
			case 1:
				emmc->version = MMC_VERSION_1_4;
				break;
			case 2:
				emmc->version = MMC_VERSION_2_2;
				break;
			case 3:
				emmc->version = MMC_VERSION_3;
				break;
			case 4:
				emmc->version = MMC_VERSION_4;
				break;
			default:
				emmc->version = MMC_VERSION_1_2;
				break;
		}
	}

	/* divide frequency by 10, since the mults are 10x bigger */
	freq = emmc_fbase[(cmd.response[0] & 0x7)];
	mult = emmc_multipliers[((cmd.response[0] >> 3) & 0xf)];

	emmc->tran_speed = freq * mult;
       printf("[%s] emmc->tran_speed = %d \n", __func__, emmc->tran_speed);
	emmc->read_bl_len = 1 << ((cmd.response[1] >> 16) & 0xf);
       printf("[%s] emmc->read_bl_len = %d \n", __func__, emmc->read_bl_len);
	if (IS_SD(emmc))
		emmc->write_bl_len = emmc->read_bl_len;
	else
		emmc->write_bl_len = 1 << ((cmd.response[3] >> 22) & 0xf);
       printf("[%s] emmc->high_capacity = %d \n", __func__, emmc->high_capacity);
	if (emmc->high_capacity) {
		csize = ((emmc->csd[1] & 0x3f) << 16)|((emmc->csd[2] & 0xffff0000) >> 16);
		cmult = 8;
		printf("[%s]1 cmult = 0x%x, csize = 0x%x \n", __func__, cmult, csize);
		emmc->capacity = (csize + 1) /2;
	} else {
		uint temp;
		csize = ((emmc->csd[1] & 0x3ff) << 2)|((emmc->csd[2] & 0xc0000000) >> 30);
		cmult = (emmc->csd[2] & 0x00038000) >> 15;
		printf("[%s]2 cmult = 0x%x, csize = 0x%x \n", __func__, cmult, csize);

		emmc->capacity = (csize + 1) << (cmult + 2);
		temp = (1024*1024)/(emmc->read_bl_len);
		emmc->capacity = emmc->capacity/temp;
	}
       
	printf("[%s] emmc->capacity = %d MB\n", __func__, emmc->capacity);
	
       
	if (emmc->read_bl_len > 512)
		emmc->read_bl_len = 512;

	if (emmc->write_bl_len > 512)
		emmc->write_bl_len = 512;

	/* Select the card, and put it into Transfer Mode */
	cmd.cmdidx = MMC_CMD_SELECT_CARD;
	cmd.resp_type = MMC_RSP_R1b;
	cmd.cmdarg = emmc->rca << 16;
	cmd.flags = 0;
	err = emmc_send_cmd(emmc, &cmd, NULL);

	if (err)
	{
	   printf("[%s] Select Card error !!\n", __func__);
		return err;
      }

	if (IS_SD(emmc))
		err = emmc_sd_change_freq(emmc);
	else
		err = emmc_change_freq(emmc);

	if (err)
	{
	   printf("[%s] Change Freq error !!\n", __func__);
		return err;
   }

	/* Restrict card's capabilities by what the host can do */
	emmc->card_caps &= emmc->host_caps;

	if (IS_SD(emmc)) {
		if (emmc->card_caps & MMC_MODE_4BIT) {
			cmd.cmdidx = MMC_CMD_APP_CMD;
			cmd.resp_type = MMC_RSP_R1;
			cmd.cmdarg = emmc->rca << 16;
			cmd.flags = 0;

			err = emmc_send_cmd(emmc, &cmd, NULL);

			if (err)
        	{
        	   printf("[%s] App Cmd error !!\n", __func__);
				return err;
          }

			cmd.cmdidx = SD_CMD_APP_SET_BUS_WIDTH;
			cmd.resp_type = MMC_RSP_R1;
			cmd.cmdarg = 2;
			cmd.flags = 0;
			err = emmc_send_cmd(emmc, &cmd, NULL);

			if (err)
        	{
        	   printf("[%s] Set bus width error !!\n, __func__");
				return err;
          }

			emmc_set_bus_width(emmc, 4);
		}

		if (emmc->card_caps & MMC_MODE_HS)
			emmc_set_clock(emmc, 50000000);
		else
			emmc_set_clock(emmc, 25000000);
	} else {
		if (emmc->card_caps & MMC_MODE_4BIT) {
			/* Set the card to use 4 bit*/
			err = emmc_switch(emmc, EXT_CSD_CMD_SET_NORMAL,
					EXT_CSD_BUS_WIDTH,
					EXT_CSD_BUS_WIDTH_4);

			if (err)
        	{
        	   printf("[%s] switch 4 bit error !!\n", __func__);
				return err;
          }

			emmc_set_bus_width(emmc, 4);
		} else if (emmc->card_caps & MMC_MODE_8BIT) {
			/* Set the card to use 8 bit*/
			err = emmc_switch(emmc, EXT_CSD_CMD_SET_NORMAL,
					EXT_CSD_BUS_WIDTH,
					EXT_CSD_BUS_WIDTH_8);

			if (err)
        	{
        	   printf("[%s] switch 8 bit error !!\n", __func__);
				return err;
          }

			emmc_set_bus_width(emmc, 8);
		}

		if (emmc->card_caps & MMC_MODE_HS) {
			if (emmc->card_caps & MMC_MODE_HS_52MHz)
				emmc_set_clock(emmc, 52000000);
			else
				emmc_set_clock(emmc, 26000000);
		} else
			emmc_set_clock(emmc, 20000000);
	}

	/* fill in device description */
/*	emmc->block_dev.lun = 0;
	emmc->block_dev.type = 0;
	emmc->block_dev.blksz = emmc->read_bl_len;
	emmc->block_dev.lba = lldiv(emmc->capacity, emmc->read_bl_len);
	sprintf(emmc->block_dev.vendor, "Man %06x Snr %08x", emmc->cid[0] >> 8,
			(emmc->cid[2] << 8) | (emmc->cid[3] >> 24));
	sprintf(emmc->block_dev.product, "%c%c%c%c%c", emmc->cid[0] & 0xff,
			(emmc->cid[1] >> 24), (emmc->cid[1] >> 16) & 0xff,
			(emmc->cid[1] >> 8) & 0xff, emmc->cid[1] & 0xff);
	sprintf(emmc->block_dev.revision, "%d.%d", emmc->cid[2] >> 28,
			(emmc->cid[2] >> 24) & 0xf);
	init_part(&emmc->block_dev);
*/
   printf("[%s] done. \n", __func__);


  // emmc_erase_block(emmc, 0, 0xffffffff);
  //	emmc_boot_wirte_debug(emmc);
 // 	emmc_boot_read_debug(emmc);
	return 0;
}

static int emmc_send_if_cond(struct emmc *emmc)
{
	struct emmc_cmd cmd;
	int err;

	cmd.cmdidx = SD_CMD_SEND_IF_COND;
	/* We set the bit if the host supports voltages between 2.7 and 3.6 V */
	cmd.cmdarg = ((emmc->voltages & 0xff8000) != 0) << 8 | 0xaa;
	cmd.resp_type = MMC_RSP_R7;
	cmd.flags = 0;

	err = emmc_send_cmd(emmc, &cmd, NULL);
   
	if (err)
		return err;

	if ((cmd.response[0] & 0xff) != 0xaa)
		return EMMC_UNUSABLE_ERR;
	else
		emmc->version = SD_VERSION_2;

	return 0;
}

static void print_emmcinfo(void)
{
	struct emmc *emmc = find_emmc_device(0);
	
	printf("[%s] Device: %s\n", __func__, emmc->name);
	printf("[%s] Manufacturer ID: 0x%x\n", __func__, emmc->cid[0] >> 24);
	printf("[%s] CARD/BGA ID: 0x%x\n", __func__, ((emmc->cid[0] >> 16)&0x0003));
	printf("[%s] OEM: 0x%x\n", __func__, (emmc->cid[0] >> 8) & 0xffff);
	printf("[%s] Name: %c%c%c%c%c \n", __func__, emmc->cid[0] & 0xff,
			(emmc->cid[1] >> 24), (emmc->cid[1] >> 16) & 0xff,
			(emmc->cid[1] >> 8) & 0xff, emmc->cid[1] & 0xff);
       printf("[%s] product SN: 0x%x\n", __func__, (((emmc->cid[2] << 16) & 0xffff0000)|((emmc->cid[3] >> 16)& 0xffff)));
	printf("[%s] Tran Speed: %d\n", __func__, emmc->tran_speed);
	printf("[%s] Rd Block Len: %d\n", __func__, emmc->read_bl_len);

	printf("[%s] %s version %d.%d\n", __func__, IS_SD(emmc) ? "SD" : "MMC",
			(emmc->version >> 4) & 0xf, emmc->version & 0xf);

	printf("[%s] High Capacity: %s\n", __func__, emmc->high_capacity ? "Yes" : "No");
	printf("[%s] Capacity: %lld MBytes\n", __func__, emmc->capacity);

	printf("[%s] Bus Width: %d-bit\n", __func__, emmc->bus_width);
}


int emmc_register(struct emmc *emmc)
{
	/* Setup the universal parts of the block interface just once */
	//emmc->block_dev.if_type = IF_TYPE_MMC;
	//emmc->block_dev.dev = 1;
	//emmc->block_dev.removable = 0;
	//emmc->block_dev.block_read = emmc_bread;
	//emmc->block_dev.block_write = emmc_bwrite;

	emmc_ptr = emmc;
	return 0;
}

static int emmc_init(void)
{
	struct emmc *emmc;
	int err;
	int i;
	int fgSD20Card = 0;


	emmc = find_emmc_device(0);
	if(emmc == NULL)
		return (EMMC_INIT_FAIL);

	err = emmc->init(emmc);

	if (err)
		return err;

        emmc_set_clock(emmc, 1);
	emmc_set_bus_width(emmc, 1);

	/* Reset the Card */
    for (i=0; i<5; i++)
    {
	err = emmc_go_idle(emmc);

	if (err)
		return err;
    }

	/* Test for SD version 2 */
	err = emmc_send_if_cond(emmc);

    /* Reset the Card if not SDHC */
    if (err)
    {    	
        for (i=0; i<5; i++)
        {
            err = emmc_go_idle(emmc);

            if (err)
                return err;
        }
    }
    else
    {
        printf("[%s] Found SD version 2.0 card ^^\n", __func__);
        fgSD20Card = 1;
    }

	/* Now try to get the SD card's operating condition */
	err = emmc_sd_send_op_cond(emmc);

	/* If the command timed out, we check for an MMC card */
    if (err == EMMC_TIMEOUT) 
    {
        /* Reset the Card if not SD */
        for (i=0; i<5; i++)
        {
            err = emmc_go_idle(emmc);

            if (err)
                return err;
        }

		err = emmc_send_op_cond(emmc);

        if (err) 
        {
            printf("No Card is detected, Card did not respond to voltage select!\n");
			return EMMC_UNUSABLE_ERR;
		}
        else
        {
            printf("[%s] Found MMC card ^^\n", __func__);
        }
    }
    else
    {
        if (!fgSD20Card)
        {
            printf("[%s] Found SD card ^^\n", __func__);
        }
	}

	return emmc_startup(emmc);
}

#if 1
extern INT32 readDataFromEMMC(UINT32 noffset, UINT32 size, void* buffer) ;
extern INT32 writeDataToEMMC(UINT32 noffset, UINT32 size, void* buffer);
void emmc_test_debug(void)
{
	int err1, err2;
	UINT8 *buff;
	//UINT32 *u4Buff;
	UINT32 i, j, length, start_LAB;
	UINT32 offset  = 0x15;
	
	printf("[%s] ------test start-------------\n", __func__);
	
	length = 0x1000;
	
	buff = malloc(length*2);
	if(!buff){
		printf("[%s] buff = NULL\n", __func__);
		return -1;
	}
	for(i=0; i<length*2; i ++)buff[i] = 0xaa;

	// 1. addr: Y, length: Y
	start_LAB = 0x0;
	err1 = writeDataToEMMC(start_LAB, length, buff);
	for(i=0; i<length*2; i ++)buff[i] = 0xa5;
	err2 = readDataFromEMMC(start_LAB, length, buff);
	if((err1&err2) != EMMC_OK){
		printf("[%s] emmc_bread 1: err1 = %d \n", __func__, err1);
		return;
	}	
	for(i=0; i<length; i ++){
		if(buff[i] != 0xaa){
			printf("[%s] compare error 1, i=%d\n", __func__, i);
			return;
		}
	}	
	printf("[%s] \n", __func__);
	
	// 2. addr: Y, length: N
	for(i=0; i<length*2; i ++)buff[i] = 0xBB;

	start_LAB = 0x1000;
	err1 = writeDataToEMMC(start_LAB, length+offset, buff);
	for(i=0; i<length*2; i ++)buff[i] = 0xa5;
	err2 = readDataFromEMMC(start_LAB, length+offset, buff);
	if((err1&err2) != EMMC_OK){

		printf("[%s] emmc_bread 2: err = %d \n", __func__, err1);
		return;
	}	
	for(i=0; i<(length+offset); i ++){
		if(buff[i] != 0xBB){
			printf("[%s] compare error 2, i=%d \n", __func__, i);
			return;
		}
	}
	printf("[%s] \n", __func__);
	
	// 3. addr: N, length: N
	for(i=0; i<length*2; i ++)buff[i] = 0xcc;
	start_LAB = 0x2000 + offset*3;
	err1 = writeDataToEMMC(start_LAB, length+offset, buff);
	for(i=0; i<length*2; i ++)buff[i] = 0xa5;
	err2 = readDataFromEMMC(start_LAB, length+offset, buff);
	if((err1&err2) != EMMC_OK){

		printf("[%s] emmc_bread 3: err = %d \n", __func__, err1);
		return;
	}
	for(i=0; i<(length+offset); i ++){
		if(buff[i] != 0xcc){
			printf("[%s] compare error 3, i=%d \n", __func__, i);
			return;
		}
	}
	printf("[%s] \n", __func__);
	
	// 4. addr: N, length: N
	for(i=0; i<length*2; i ++)buff[i] = 0xdd;
	start_LAB = 0x4000 + offset*3;
	err1 = writeDataToEMMC(start_LAB, length+offset, buff);
	for(i=0; i<length*2; i ++)buff[i] = 0xa5;
	err2 = readDataFromEMMC(start_LAB, length+offset, buff);
	if((err1&err2) != EMMC_OK){

		printf("[%s] emmc_bread 4: err = %d \n", __func__, err1);
		return;
	}
	for(i=0; i<(length+offset); i ++){
		if(buff[i] != 0xdd){
			printf("[%s] compare error 4, i=%d \n", __func__, i);
			return;
		}
	}
	printf("[%s] \n", __func__);
	printf("[%s] ------test end-------------\n", __func__);
	return;
}
#endif

/*-----------------------------------------------------------------------------
 * Name: emmc_initialize
 * Description: eMMC card initialize and detect.
 *
 * Inputs:    	

 * Returns: 	EMMC_OK		eMMC card is detected.
 *			EMMC_FAIL		eMMC card is not existed.
 ----------------------------------------------------------------------------*/
int emmc_initialize(void)
{
	int err;
	
	err = emsdc_mmc_init();
	if(err)
		return err;

	err = emmc_init();
	if(err)
		return err;
	
	print_emmcinfo();

	//emmc_test_debug();
	//while(1);

	return ((err)? EMMC_FAIL : EMMC_OK);
}
