/*
 *  linux/drivers/emmc/mtk_sd.c - Secure Digital Host Controller Interface driver
 *
 *  Copyright (C) 2009 Mediatek.Inc, All Rights Reserved.
 *
 */
#include <config.h>
#include <common.h>
#include <command.h>
#include <hwconfig.h>

#include <part.h>
#include <malloc.h>

#include <asm/io.h>
#include <asm/arch/x_typedef.h>
#include <asm/arch/mt85xx.h>

#include <mt85xx_gpio.h>

#include <emmc.h>
#include "emmc_msdc.h"
#include "emsdchw_reg.h"
#include "emsdchw_io.h"
#include "emsdc_const.h"
#include "e_x_ckgen.h"


#define DRIVER_NAME "EMMC_MSDC"
#define DRIVER_VERSION "0.10"

#define BUGMAIL "Mediatek. Inc"

DECLARE_GLOBAL_DATA_PTR;

/*------------------------------------------------------------------------------
    LOG Debug function
------------------------------------------------------------------------------*/
#ifdef EMSDC_LOG_ENABLE
#define EMSDC_LOG_ERROR                       ((u32)1 << 0)
#define EMSDC_LOG_CARD                         ((u32)1 << 1)
#define EMSDC_LOG_REQ                           ((u32)1 << 2)
#define EMSDC_LOG_CLK                           ((u32)1 << 3)
#define EMSDC_LOG_RETRY                       ((u32)1 << 4)
	
#define EMSDC_LOG_INIT                          ((u32)1 << 5)
#define EMSDC_LOG_FUNC                        ((u32)1 << 6)
#define EMSDC_LOG_IRQ                           ((u32)1 << 8)
#define EMSDC_LOG_SDHWCMD                 ((u32)1 << 9)
#define EMSDC_LOG_IOCMD                       ((u32)1 << 10)
#define EMSDC_LOG_RW                            ((u32)1 << 11)
#define EMSDC_LOG_BUFFER                     ((u32)1 << 12)
#define EMSDC_LOG_DBG                          ((u32)1 << 13)
	
static unsigned int emsdc_log_en =  EMSDC_LOG_RETRY | EMSDC_LOG_ERROR | EMSDC_LOG_CARD | EMSDC_LOG_CLK;

#define EMSDC_LOG(n, f, x...) if (emsdc_log_en & (n)){ printf(DRIVER_NAME ": <%s	> " f, __func__,## x); }
//#define FCI_LOG(n, f, x...) if (fci_log_en & (n)){ printf(DRIVER_NAME ": <%s> " f, __func__,## x); }

#else
#define EMSDC_LOG(n, f, x...)
#endif


/*------------------------------------------------------------------------------
    Static variables
------------------------------------------------------------------------------*/

static struct mtk_emsdc_host emsdc_host;

volatile unsigned int eMsdcAccuVect = 0x00;

//static UINT32 u4emsdcFreqIdx = 0;
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
static UINT32 u4eMsdcFreq[] = {INIT_BUS_FREQ, EMSDC_CLK_3375K, EMSDC_CLK_13_5M,EMSDC_CLK_23M, EMSDC_CLK_27M, BUS_CLK_FREQ};
#endif

/*------------------------------------------------------------------------------
    Global variables
------------------------------------------------------------------------------*/

//BOOL _bFCICardExist = FALSE;

/*------------------------------------------------------------------------------
    Imported Global variables
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
    Module Parameter variable Entry
------------------------------------------------------------------------------*/
static INT32 _u4eSDMultiFuncCfg = E_SD_MLT_FNC_CFG;
static INT32 _u4eSDPwrCdPinCfg = E_SD_PWR_CD_PIN_CFG;
static INT32 _u4eSDSdWpPinCfg = E_SD_WP_PIN_CFG;

/*------------------------------------------------------------------------------
    Static  functions
------------------------------------------------------------------------------*/
static int eMsdcWaitClkStable (UINT32 ch);
static VOID eMsdcSDDrivingCurrent (INT32 i4ClkMHz);
static VOID CKGEN_AgtSelClk(UINT32 ch, UINT32 u4Sel);


/*-------------------------g-----------------------------------------------------
    Imported Global functions
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
    EMSDC base functions
------------------------------------------------------------------------------*/
// ---------------------------------
// Return BOOL : TRUE - card exist
// Return BOOL : FALSE - card not exist
// ---------------------------------
int emsdc_card_exist(void)
{
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)

  if (_u4eSDPwrCdPinCfg == 1)
  {
	GPIO_Config(74, 0, 1);  //config PIN_VOUT7 as input mode
	EMSDC_LOG(EMSDC_LOG_FUNC, "card_exist: %d \n", GPIO_Input(74));
	return ((GPIO_Input(74) ? 0 : 1));
  }
#endif

  return FALSE;
}

//-------------------------------------------------------------------------
/** eMsdcSetClkfreq
 *  Set EMSDC clock divide
 *  @param  ch : channel
 *  @param  clkFreq : Target frequency setting (in Hz)
 *  @return  void
 */
//-------------------------------------------------------------------------
static int eMsdcSetClkfreq (UINT32 ch, unsigned int clkFreq)
{
 // unsigned int divider;
 // unsigned int expFreq = 0;
/*  
  if ((clkFreq > BUS_CLK_FREQ))
  {
    clkFreq = BUS_CLK_FREQ;
    EMSDC_LOG(EMSDC_LOG_ERROR, "%s: over max frequency setting, set clkFreq : %d Hz\r\n", __FUNCTION__, clkFreq);
  }
  else if ((clkFreq < LOWEST_BUS_FREQ))
  {
    clkFreq = INIT_BUS_FREQ;
    EMSDC_LOG(EMSDC_LOG_ERROR, "%s: set clkFreq : %d Hz\r\n", __FUNCTION__, clkFreq);
  }
*/
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
  CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL1_1_8);
  x_thread_delay(2);
  
  switch (clkFreq)
  {
    case EMSDC_CLK_54M:
   //   _u4SDClockSetting = 54;
      // 0x7000_0078 bit[14:8] = xxx_0001, SD_CLK = 54 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL1_1_8);
      break;
  
    case EMSDC_CLK_50M:
  //    _u4SDClockSetting = 50;
      // 0x7000_0078 bit[14:8] = xxx_0100, SD_CLK = 50 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL2_1_12);
      break;
  
    case EMSDC_CLK_36M:
     // _u4SDClockSetting = 36;
      // 0x7000_0078 bit[14:8] = xxx_0011, SD_CLK = 36 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL1_1_12);
      break;
			  
    case EMSDC_CLK_27M:
    //  _u4SDClockSetting = 27;
      // 0x7000_0078 bit[14:8] = xxx_0000, SD_CLK = 27 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M);
      break;

    case EMSDC_CLK_23M:
    //  _u4SDClockSetting = 23;
      // 0x7000_0078 bit[14:8] = xxx_0111, SD_CLK = 23.08 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_SYSPLL2_1_26);
      break;
						  
    case EMSDC_CLK_13_5M:
     // _u4SDClockSetting = 13;
      // 0x7000_0078 bit[14:8] = 000_1001, SD_CLK = 13.50 MHz OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M_1_2);
      break;   	   
			  
    case EMSDC_CLK_3375K:
     // _u4SDClockSetting = 3;
      //  0x7000_0078 bit[14:8] = 010_1001, SD_CLK = 3.38 MHz  OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M_1_8);
      break; 	   
			  
    case EMSDC_CLK_420K:
    default:
     // _u4SDClockSetting = 0;
      //  0x7000_0078 bit[14:8] = 101_1001, SD_CLK = 421.885 KHz  OK
      CKGEN_AgtSelClk(ch, CLK_CLK_SD_SEL_27M_1_64);
      // 0x7000_0078 bit[14:8] = 110_1001, SD_CLK = 210.94 KHz	OK
     //	CKGEN_AgtSelClk(e_CLK_SD, CLK_CLK_SD_SEL_27M_1_128);
     break;
  
     // For reference
     // 0x7000_0078 bit[14:8] = 100_1001, SD_CLK = 843.75 KHz	OK
     // 0x7000_0078 bit[14:8] = 110_1001, SD_CLK = 210.94 KHz	OK
     // 0x7000_0078 bit[14:8] = 111_1001, SD_CLK = 105.47 KHz	OK

  } 
  x_thread_delay(1);
#endif
//  EMSDC_WRITE32(EMSDC_CFG(ch), (EMSDC_READ32(EMSDC_CFG(ch)) & (~0x0000FF00)));

  // Wait until clock is stable
  if (FAILED == eMsdcWaitClkStable(ch))
  {
     return FAILED;
  }
  
  EMSDC_LOG(EMSDC_LOG_CLK, "Clock set OK! Clk Frequency : %d Hz\r\n", clkFreq);
  
  // Setup driving current according to working frequency
  eMsdcSDDrivingCurrent (clkFreq/1000);
  
  emsdc_host.clk = clkFreq;
  
  return SUCCESS;


/*
  for (divider = 0; divider <= 0xFF; divider++)
  {
    if (0 == divider)
    {
      expFreq = SRC_CLK_FREQ / 2;
    }
    else
    {
      expFreq = SRC_CLK_FREQ / 4 / divider;
    }

    if (expFreq <= clkFreq)
    {
      EMSDC_WRITE32(EMSDC_CFG(ch), (EMSDC_READ32(EMSDC_CFG(ch)) & (~0x0000FF00)) | (divider << EMSDC_CFG_CK_DIV_SHIFT));

      // Wait until clock is stable
      if (FAILED == eMsdcWaitClkStable(ch))
      {
        return FAILED;
      }

      EMSDC_LOG(EMSDC_LOG_CLK, "%s: Divider 0x%02X, clkFreq : %d, Expect Frequency : %d Hz\r\n", __FUNCTION__, divider, clkFreq, expFreq);

      // Setup driving current according to working frequency
      eMsdcSDDrivingCurrent (expFreq/1000);

      emsdc_host->clk = clkFreq;
      
      return SUCCESS;
    }
  }
*/
  EMSDC_LOG(EMSDC_LOG_ERROR, "Failed to search correct divider.\r\n");

  return FAILED;
}


static void emsdc_set_clock(struct mtk_emsdc_host *host, UINT32 dest_clk)
{
  unsigned int clk;

  UINT32 i, level;

  level = sizeof(u4eMsdcFreq) /sizeof(UINT32);
  for(i=0; i<level; i++)
  {
    if(dest_clk <= u4eMsdcFreq[0])
  {
    clk = u4eMsdcFreq[0];
       break;
    }
    else if(dest_clk >= u4eMsdcFreq[level-1])
    {
       clk = u4eMsdcFreq[level-1];
       break;
    }
    else if(dest_clk < u4eMsdcFreq[i])
    {
       clk = u4eMsdcFreq[i-1];
       break;
    }
  }

  // Only Test: clock is always output
 // eMsdcContinueClock(0, 1);
	
  EMSDC_LOG(EMSDC_LOG_CLK, "level=%d, Dest_Clk = %dhz, set_clk = %d hz\n", level, dest_clk, clk);

 /*
   * Only write to the clock register when
   * there is an actual change.
   */
  if (clk != host->clk)
  {
    eMsdcSetClkfreq(0, clk);
  }
}

//-------------------------------------------------------------------------
/** eMsdcSetBusWidth
 *  eMsdc hardware bus width setting function
 *  @param  ch : channel
 *  @param  busWidth : 1 or 4 bit
 *  @return  void
 */
//-------------------------------------------------------------------------
static int eMsdcSetBusWidth (UINT32 ch, int busWidth)
{
  /* Modify EMSDC Register Settings */
  if (1 == busWidth)
  {
    EMSDC_WRITE32(E_SDC_CFG(ch), (EMSDC_READ32(E_SDC_CFG(ch)) & ~E_SDC_CFG_BW_MASK) | E_SDC_CFG_BW_1BIT);
  }
  else if (4 == busWidth)
  {
    EMSDC_WRITE32(E_SDC_CFG(ch), (EMSDC_READ32(E_SDC_CFG(ch)) & ~E_SDC_CFG_BW_MASK) | E_SDC_CFG_BW_4BIT);
  }

  return SUCCESS;
}

static void emsdc_set_ios(struct emmc *emmc)
{
  int ch = 0;
   
  /* Set the clock speed */	
  emsdc_set_clock(emmc->priv, emmc->clock);


  /* Set the bus width */
  if ( emsdc_host.bus_width != emmc->bus_width)
  {
    emsdc_host.bus_width = emmc->bus_width;
		
    EMSDC_LOG(EMSDC_LOG_CARD, "Switch to %d Bit Mode.\n", emmc->bus_width); 
    eMsdcSetBusWidth(ch, emmc->bus_width);
  }
}


/*------------------------------------------------------------------------------
    EMSDC Low Level functions
------------------------------------------------------------------------------*/
//-------------------------------------------------------------------------
/** eMsdcReset
 *  Reset eMsdc HW (register value will not be reset)
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int eMsdcReset (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  // Reset EMSDC
  EMSDC_SETBIT(E_MSDC_CFG(ch), E_MSDC_CFG_RST);

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (0 == (EMSDC_READ32(E_MSDC_CFG(ch)) & E_MSDC_CFG_RST))
    {
      break;
    }
    else if (u4DiffTime >= EMSDC_RST_TIMEOUT_LIMIT)
    {
      EMSDC_LOG(EMSDC_LOG_ERROR, "timeout error !\n");
      return FAILED;
    }

    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}

//-------------------------------------------------------------------------
/** eMsdcClrFifo
 *  Clear eMsdc FIFO status.
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int eMsdcClrFifo (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

//  EMSDC_WAIT_SETBIT(EMSDC_FIFOCS(ch), EMSDC_FIFOCS_FIFOCLR);

  // Reset FIFO
  EMSDC_SETBIT(E_MSDC_FIFOCS(ch), E_MSDC_FIFOCS_FIFOCLR);

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (0 == (EMSDC_READ32(E_MSDC_FIFOCS(ch)) & (E_MSDC_FIFOCS_FIFOCLR | E_MSDC_FIFOCS_TXFIFOCNT_MASK | E_MSDC_FIFOCS_RXFIFOCNT_MASK)))
    {
      break;
    }
    else if (u4DiffTime >= EMSDC_FIFOCLR_TIMEOUT_LIMIT)
    {
      EMSDC_LOG(EMSDC_LOG_ERROR, "timeout error !\n");
      return FAILED;
    }

    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}

//-------------------------------------------------------------------------
/** eMsdcChkFifo
 *  Check if eMsdc FIFO = 0.
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static void eMsdcChkFifo (UINT32 ch)
{
  // Check if rx/tx fifo is zero
  if ((EMSDC_READ32(E_MSDC_FIFOCS(ch)) & (E_MSDC_FIFOCS_TXFIFOCNT_MASK | E_MSDC_FIFOCS_RXFIFOCNT_MASK)) != 0)
  {
    EMSDC_LOG(EMSDC_LOG_ERROR, "FiFo not 0, FIFOCS:0x%08X !!\n", EMSDC_READ32(E_MSDC_FIFOCS(ch)));
    eMsdcClrFifo(ch);
  }
}

//-------------------------------------------------------------------------
/** eMsdcClrIntr
 *  Clear EMSDC interrupt vector
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static void eMsdcClrIntr (UINT32 ch)
{
  // Check EMSDC Interrupt vector register
  if  (0x00  != EMSDC_READ32(E_MSDC_INT(ch)))
  {
    EMSDC_LOG(EMSDC_LOG_ERROR, "EMSDC INT not 0:0x%08X !!\r\n", EMSDC_READ32(E_MSDC_INT(ch)));

    // Clear EMSDC Interrupt vector register
    EMSDC_WRITE32(E_MSDC_INT(ch), EMSDC_READ32(E_MSDC_INT(ch)));
  }
}


//-------------------------------------------------------------------------
/** eMsdcWaitClkStable
 *  Wait EMSDC Clock stable after Mode/DIV changed
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int eMsdcWaitClkStable (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms
    if (0 != (EMSDC_READ32(E_MSDC_CFG(ch)) & E_MSDC_CFG_CARD_CK_STABLE))
    {
      break;
    }
    else if (u4DiffTime >= EMSDC_CLK_TIMEOUT_LIMIT)
    {
      EMSDC_LOG(EMSDC_LOG_ERROR, "WaitClkStable Failed !\r\n");
      return FAILED;
    }
    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}

//-------------------------------------------------------------------------
/** eMsdcContinueClock
 *  Set continue clock output or not
 *  @param  ch : channel
 *  @param  i4ContinueClock TRUE - Continue clock, FALSE - clock output when command is sending
 *  @return  void
 */
//-------------------------------------------------------------------------
static void eMsdcContinueClock (UINT32 ch, int i4ContinueClock)
{
  if (i4ContinueClock)
  {
    // Set clock continuous even if no command
    EMSDC_SETBIT(E_MSDC_CFG(ch), (((UINT32)0x01) << 1));
  }
  else
  {
    // Set clock power down if no command
    EMSDC_CLRBIT(E_MSDC_CFG(ch), (((UINT32)0x01) << 1));
  }
}

//-------------------------------------------------------------------------
/** eMsdcHWConfigSetup
 *  Setup multi-func, driving current, pull up resistor ... HW setting.
 *  @return  VOID.
 */
//-------------------------------------------------------------------------
static VOID eMsdcHWConfigSetup(void)
{
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)

#endif
}

//-------------------------------------------------------------------------
/** eMsdcSDDrivingCurrent
 *  Set SD driving current according to clock
 *  @param i4ClkMHz    clock setting (Mhz).
 *  @return  VOID
 */
//-------------------------------------------------------------------------

static VOID eMsdcSDDrivingCurrent (INT32 i4ClkMHz)
{
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
  switch(_u4eSDMultiFuncCfg)
  {
    case 1:
      {
        // Driving current setting
        if (i4ClkMHz >= 27000000)
        {
          // Set driving current E4+E8
        }
        else
        {
          // Set driving current E2
        }
      }
      break;
  }
#endif
}

//-------------------------------------------------------------------------
/** CKGEN_AgtEnableClk
 *  Turn on / off SD dram clock
 *  @param fgEnable    on / off
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID CKGEN_AgtEnableClk(UINT32 ch, BOOL fgEnable)
{
  UINT32 u4Tmp;

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) 
  u4Tmp = CKGEN_READ32(REG_RW_CLK_CFG2);

  if(fgEnable)
    u4Tmp = u4Tmp & (~CLK_PDN_SD);
  else
    u4Tmp = u4Tmp | CLK_PDN_SD;
  
  CKGEN_WRITE32(REG_RW_CLK_CFG2, u4Tmp);
  EMSDC_LOG(EMSDC_LOG_ERROR, "Enable EMSDC dram Clk !\n");
  x_thread_delay(1);
  
#endif

    return;
}

//-------------------------------------------------------------------------
/** CKGEN_AgtSelClk
 *  set dram clock
 *  @param u4Sel : offset
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID CKGEN_AgtSelClk(UINT32 ch, UINT32 u4Sel)
{
    UINT32 u4Tmp;
    
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) 
  u4Tmp = CKGEN_READ32(REG_RW_CLK_CFG2);

  u4Sel = (u4Sel << CLK_CLK_SD_SEL_OFFSET) & CLK_CLK_SD_SEL_MASK;
  u4Tmp = u4Tmp & (~CLK_CLK_SD_SEL_MASK);
  u4Tmp = u4Tmp | u4Sel;
              
  CKGEN_WRITE32(REG_RW_CLK_CFG2, u4Tmp);

#endif

    return;    
}



//-------------------------------------------------------------------------
/** eMsdcInit
 *  eMsdc hardware Initialize function
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int eMsdcInit (UINT32 ch)
{
  CKGEN_AgtEnableClk(ch, TRUE);

  // Reset EMSDC
  eMsdcReset(ch);

  // Set DMA Mode & SD/MMC Mode  
  EMSDC_CLRBIT(E_MSDC_CFG(ch), E_MSDC_CFG_PIO_MODE);
  EMSDC_SETBIT(E_MSDC_CFG(ch), E_MSDC_CFG_DMA_MODE | E_MSDC_CFG_SD);

  // Disable sdio & Set bus to 1 bit mode
  EMSDC_CLRBIT(E_SDC_CFG(ch), E_SDC_CFG_SDIO | E_SDC_CFG_BW_MASK);

  // set clock mode (Direct mode)
  EMSDC_CLRBIT(E_MSDC_CFG(ch), (((UINT32)0x03) << 16));
  EMSDC_SETBIT(E_MSDC_CFG(ch), E_MSDC_CFG_CK_MODE_DIRECT);

  // Wait until clock is stable
  if (FAILED == eMsdcWaitClkStable(ch))
  {
    return FAILED;
  }

  // Set default RISC_SIZE for DWRD pio mode
  EMSDC_WRITE32(E_MSDC_IOCON(ch), (EMSDC_READ32(E_MSDC_IOCON(ch)) & ~E_MSDC_IOCON_RISC_SIZE_MASK) | E_MSDC_IOCON_RISC_SIZE_DWRD);

  // Set Data timeout setting => Maximum setting
  EMSDC_WRITE32(E_SDC_CFG(ch), (EMSDC_READ32(E_SDC_CFG(ch)) & ~(((UINT32)0xFF) << E_SDC_CFG_DTOC_SHIFT)) | (((UINT32)0xFF) << E_SDC_CFG_DTOC_SHIFT));

  // Enable interrupt bit
  EMSDC_SETBIT(E_MSDC_INTEN(0), E_INT_SD_ALL);

  // Setup HW configuration for EMSDC
  eMsdcHWConfigSetup();
  
  return SUCCESS;
}



//-------------------------------------------------------------------------
/** eMsdcPowerOn
 *  Power on EMSDC Clock
 *  @param  ch : channel
 *  @return TRUE.
 */
//-------------------------------------------------------------------------
static void eMsdcPowerOn (UINT32 ch)
{
  // Card CK Drive enable
  EMSDC_SETBIT(E_MSDC_CFG(ch), E_MSDC_CFG_CK_DRV_EN);

  CKGEN_AgtEnableClk(ch, TRUE);
  
}

//-------------------------------------------------------------------------
/** eMsdcPowerOff
 *  Power off EMSDC Clock
 *  @param  ch : channel
 *  @return TRUE.
 */
//-------------------------------------------------------------------------
static void eMsdcPowerOff (UINT32 ch)
{
  // Card CK Drive enable
  EMSDC_CLRBIT(E_MSDC_CFG(ch), E_MSDC_CFG_CK_DRV_EN);

  CKGEN_AgtEnableClk(ch, FALSE);
}

//-------------------------------------------------------------------------
/** eMsdcDmaEnable
 *  Enable EMSDC DMA for read/write operation
 *  @param ch            channel
 *  @param pu1Buf      user's read buffer pointer.
 *  @param u4BlkSz     write block size in bytes.
 *  @param fgWrite     TRUE for write, FALSE for read.
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID eMsdcDmaEnable (UINT32 ch, UINT8 *pu1Buf, UINT32 u4BlkSz, BOOL fgWrite)
{
  ASSERT(pu1Buf);

//  BSP_FlushDCacheRange((UINT32)pu1Buf, u4BlkSz);

  EMSDC_WRITE32(E_DMA_SA(ch), (UINT32) pu1Buf);
  EMSDC_WRITE32(E_DMA_CTRL(ch), (u4BlkSz << E_DMA_CTRL_XFER_SIZE_SHIFT) | E_DMA_CTRL_BST_64 | E_DMA_CTRL_LAST_BUF | E_DMA_CTRL_START);
}

//-------------------------------------------------------------------------
/** eMsdcHw_DmaDisable
 *  Disable DMA transfer.
 *  @param  ch : channel
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID eMsdcDmaDisable (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  EMSDC_SETBIT(E_DMA_CTRL(ch), E_DMA_CTRL_STOP);
  
  u4JiffSt = u4JiffEnd = get_timer(0);
  
   while (1)
   {
	  u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;	// unit : ms
  
	  if (! (EMSDC_READ32(E_DMA_CTRL(ch)) & E_DMA_CTRL_STOP))
	  {
		break;
	  }
	  else if (u4DiffTime >= EMSDC_WAIT_CMD_TIMEOUT_LIMIT)
	  {
		EMSDC_LOG(EMSDC_LOG_ERROR, "timeout error !\n");
		return FAILED;
	  }
  
	  u4JiffEnd = get_timer(0);
  }

  while (EMSDC_READ32(E_DMA_CTRL(ch)) & E_DMA_CTRL_STOP);
}

//-------------------------------------------------------------------------
/** eMsdcDmaStatusChk
 *  Check DMA is in-active, for ever loop until DMA become in-active
 *  @param  ch : channel
 *  @return  VOID
 */
//-------------------------------------------------------------------------
static VOID eMsdcDmaStatusChk (UINT32 ch)
{
  while (EMSDC_READ32(E_DMA_CFG(ch)) & E_DMA_CFG_DMA_STATUS);
}

//-------------------------------------------------------------------------
/** eMsdcErrorHandling
 *  eMsdc Error handling function
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static void eMsdcErrorHandling (UINT32 ch, struct emmc_data *data)
{
   EMSDC_LOG(EMSDC_LOG_ERROR, "1 \n" );
  
  if (data)
  {
     EMSDC_LOG(EMSDC_LOG_ERROR, "2 \n" );
    // Step 2 : STOP DMA by writing 1 to DMA_CTRL.DMA_STOP and wait it becomes 0
    eMsdcDmaDisable(ch);
	EMSDC_LOG(EMSDC_LOG_ERROR, "3 \n" );

    // Step 3 : Check DMA status is 0 by reading DMA_CS.DMA_STATUS
    eMsdcDmaStatusChk(ch);    
  }
  EMSDC_LOG(EMSDC_LOG_ERROR, "4 \n" );

    // Step 1 : Reset EMSDC by writing 1 to EMSDC_CFG.RST and wait it becomes 0
  eMsdcReset(ch);
   EMSDC_LOG(EMSDC_LOG_ERROR, "5 \n" );
  // Step 4 : Clear FIFO by writing 1 to EMSDC_FIFOCS.FIFOCLR and wait it becomes 0
  eMsdcClrFifo(ch);
  EMSDC_LOG(EMSDC_LOG_ERROR, "6 \n" );

  // Step 5 : Check EMSDC_FIFOCS.TXFIFOCNT/ RXFIFOCNT are both 0
  eMsdcChkFifo(ch);
  EMSDC_LOG(EMSDC_LOG_ERROR, "7 \n" );

  // Step 6 : Make sure the E_MSDC_INT are all cleared
  eMsdcClrIntr(ch);
}

//-------------------------------------------------------------------------
/** eMsdcWaitSDCBusIdle
 *  Wait SD Bus Idle
 *  @param  ch : channel
 *  @return  void
 */
//-------------------------------------------------------------------------
static int eMsdcWaitSDCBusIdle (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;
  int ret = SUCCESS, retry = 0;
  
  retry_waitIdle:
  // Wait SDC bus idle
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms
    if ((0 == (EMSDC_READ32(E_SDC_STS(ch)) & (E_SDC_STS_SDCBUSY | E_SDC_STS_CMDBUSY))) && (0x00  == EMSDC_READ32(E_MSDC_INT(ch))))
    {
      ret = SUCCESS;
      break;
    }
    else if (u4DiffTime >= EMSDC_WAIT_SDC_BUS_TIMEOUT_LIMIT)
    {
      EMSDC_LOG(EMSDC_LOG_ERROR, "SDC_STS=0x%x, E_MSDC_INT=0x%x \n", EMSDC_READ32(E_SDC_STS(ch)), EMSDC_READ32(E_MSDC_INT(ch)));

	ret = FAILED;
	retry ++;
	break;
    }
    u4JiffEnd = get_timer(0);
  }
  
  if(retry == 1)
  {	
     retry++;
     eMsdcErrorHandling(ch, 0);
     
     goto retry_waitIdle;
  }
  return ret;
}


//-------------------------------------------------------------------------
/** eMsdcPowerSwitch
 *  Initailize all eMsdc card.
 *  @param  fgPowerOn TRUE : power on, FALSE : power off
 *  @retval   none
 */
//-------------------------------------------------------------------------
static VOID eMsdcPowerSwitch(BOOL fgPowerOn, UINT32 u4Delayms)
{
  UNUSED(fgPowerOn);

 // emsdc_host.fgPowerSwitchOn = fgPowerOn;

  EMSDC_LOG(EMSDC_LOG_RETRY, "emmc : Power %s for %d ms.\n", fgPowerOn ? "On" : "Off", u4Delayms);

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
/*
  if (_u4eSDPwrCdPinCfg == 1)
  {
	 if(fgPowerOn)    // configure GPIO low to turn on FCI power
	 {
		GPIO_Config(74, 1, 0);  //config PIN_VOUTD7 as Output mode = LOW
		GPIO_Config(74, 0, 0);  //config PIN_VOUTD7 as input mode as card detect mode
	}
	else
	{
		GPIO_Config(74, 1, 1);  //config PIN_VIND30 as Output mode = High
	}
  }
*/
  // Apply delay time
 // x_thread_delay(u4Delayms);
#endif
}

// Return when any interrupt is matched or timeout
static int eMsdcWaitIntr (unsigned int vector, unsigned int or, unsigned int timeoutCnt) 
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime=0;
  int ch = 0;
  
  u4JiffSt = u4JiffEnd = get_timer(0);  
  
  while (u4DiffTime <= timeoutCnt)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (or) 
    {
        // Status match any bit
        if (0 != (EMSDC_READ32(E_MSDC_INT(ch)) & vector)) 
        {
            return SUCCESS;
        }
    }
    else 
    {
        // status exactly match
        if (vector == EMSDC_READ32(E_MSDC_INT(ch)))
        {
            return SUCCESS;
        }
    }        
        
    u4JiffEnd = get_timer(0);
  } 

  // Timeout case
  return FAILED;
}

//-------------------------------------------------------------------------
/** eMsdcWaitCmdRdyOrTo
 *  Wait command line ready. Use in commad write operation.
 *  @param  ch : channel
 *  @retval  S_OK  Success.
 *  @retval  Others  Fail.
 */
//-------------------------------------------------------------------------
static INT32 eMsdcWaitCmdRdyOrTo(UINT32 ch, struct mtk_emsdc_host *host, struct emmc_cmd *cmd)
{
    UINT32 u4CmdDoneVect, eMsdcAccuVect;

#ifdef CC_CARD_DETECT_PIN
     // handle interrupt status only in card inserted state.
    if (!(emsdc_card_exist()))
    {
        EMSDC_LOG(EMSDC_LOG_ERROR, " No Card error.\n");
        return E_NO_CARD;
    }
#endif

    // Wait for command and response if existed
    u4CmdDoneVect = E_INT_SD_CMDRDY | E_INT_SD_CMDTO | E_INT_SD_RESP_CRCERR;

    if (SUCCESS != eMsdcWaitIntr(u4CmdDoneVect, 1, EMSDC_WAIT_CMD_TIMEOUT_LIMIT)) 
    {
        EMSDC_LOG(EMSDC_LOG_ERROR, "Failed to send CMD/RESP, DoneVect = 0x%x.\r\n", u4CmdDoneVect);
        return E_NO_RESPONSE;
    }

    eMsdcAccuVect = EMSDC_READ32(E_MSDC_INT(ch));
    EMSDC_WRITE32(E_MSDC_INT(ch), EMSDC_READ32(E_MSDC_INT(ch)));
//    eMsdcClrIntr(ch);

    EMSDC_LOG(EMSDC_LOG_IRQ, "eMsdcAccuVect = 0x%x, E_MSDC_INT = 0x%x.\r\n", eMsdcAccuVect, EMSDC_READ32(E_MSDC_INT(ch)));
    if (eMsdcAccuVect & E_INT_SD_CMDTO)
    {
        EMSDC_LOG(EMSDC_LOG_ERROR, "CMD%d ARG 0x%08X - CMD Timeout (AccuVect 0x%08X INTR 0x%08X).\r\n", cmd->cmdidx, cmd->cmdarg, eMsdcAccuVect, EMSDC_READ32(E_MSDC_INT(ch)));        
        return E_CMD_TIMEOUT;
    }
    else if (eMsdcAccuVect & E_INT_SD_RESP_CRCERR)
    {
        EMSDC_LOG(EMSDC_LOG_ERROR, "CMD%d ARG 0x%08X - CMD CRC Error (AccuVect 0x%08X INTR 0x%08X).\r\n", cmd->cmdidx, cmd->cmdarg, eMsdcAccuVect, EMSDC_READ32(E_MSDC_INT(ch)));
        return E_CMD_RSP_CRC_ERR;
    }
    else if ((eMsdcAccuVect & (~(E_INT_SD_CMDRDY))) || (0 != EMSDC_READ32(E_MSDC_INT(ch))))
    {
        EMSDC_LOG(EMSDC_LOG_ERROR, "CMD%d ARG 0x%08X - UnExpect status (AccuVect 0x%08X INTR 0x%08X).\r\n", cmd->cmdidx, cmd->cmdarg, eMsdcAccuVect, EMSDC_READ32(E_MSDC_INT(ch)));
        return E_FAIL;
    }

    return S_OK;
}

//-------------------------------------------------------------------------
/** eMsdcReadResp
 *  read SD response.
 *  @param  ch : channel
 *  @param index    response index, range in 1~4..
 *  @return  response value.
 */
//-------------------------------------------------------------------------
static UINT32 eMsdcReadResp(UINT32 ch, INT32 index)
{
  UINT32 u4Val;
  
  switch(index)
  {
    case 2:
      u4Val = EMSDC_READ32(E_SDC_RESP1(ch));
      break;
    case 3:
      u4Val = EMSDC_READ32(E_SDC_RESP2(ch));
      break;
    case 4:
      u4Val = EMSDC_READ32(E_SDC_RESP3(ch));
      break;
    case 1:
    default:
      u4Val = EMSDC_READ32(E_SDC_RESP0(ch));
      break;
  }
  return (u4Val);
}

//-------------------------------------------------------------------------
/** eMsdcWaitDataRdyOrTo
 *  Wait data line ready. Use in data read operation.
 *  @param  ch : channel
 *  @retval  S_OK  Success.
 *  @retval  Others  Fail.
 */
//-------------------------------------------------------------------------
static INT32 eMsdcWaitDataRdyOrTo(UINT32 ch, struct mtk_emsdc_host *host, struct emmc_data *data)
{
  UINT32 u4DataDoneVect, eMsdcAccuVect;
  INT32 ret = S_OK;
#ifdef CC_CARD_DETECT_PIN
     // handle interrupt status only in card inserted state.
  if (!(emsdc_card_exist()))
  {
    EMSDC_LOG(EMSDC_LOG_ERROR, " No Card error.\n");
    return E_NO_CARD;
  }
#endif  

  // Wait for command and response if existed
  u4DataDoneVect = E_INT_SD_DATA_STS;
  
  if (SUCCESS != eMsdcWaitIntr(u4DataDoneVect, 1, EMSDC_WAIT_DATA_TIMEOUT_LIMIT)) 
  {
    EMSDC_LOG(EMSDC_LOG_ERROR, "Failed to wait data IRQ, E_MSDC_INT = 0x%x.\r\n", EMSDC_READ32(E_MSDC_INT(ch)));
    return E_DATA_TIMEOUT;
  }
  
  eMsdcAccuVect = EMSDC_READ32(E_MSDC_INT(ch));
  EMSDC_WRITE32(E_MSDC_INT(ch), EMSDC_READ32(E_MSDC_INT(ch)));
  //	eMsdcClrIntr(ch);
  EMSDC_LOG(EMSDC_LOG_IRQ, "eMsdcAccuVect = 0x%x, E_MSDC_INT = 0x%x.\r\n", eMsdcAccuVect, EMSDC_READ32(E_MSDC_INT(ch)));

  if (eMsdcAccuVect & E_INT_SD_DATTO)
  {
    ret = E_DATA_TIMEOUT;
  }
  else if (eMsdcAccuVect & E_INT_SD_DATA_CRCERR)
  {
    ret = E_DATA_CRC_ERROR;
  }
  else if (eMsdcAccuVect & (E_INT_SD_XFER_COMPLETE|E_INT_DMA_XFER_DONE))
  {
    return S_OK;
  }
  else
  {    
    ret = E_FAIL;
  }  
  
  EMSDC_LOG(EMSDC_LOG_ERROR, "Data error: E_MSDC_INT=0x%x, eMsdcAccuVect=0x%x\n", EMSDC_READ32(E_MSDC_INT(ch)), eMsdcAccuVect);
  
  return ret;        
}

//-------------------------------------------------------------------------
/** eMsdcSendCmd
 *  Send SD command and argument.
 *  @param u4Cmd    SD command.
 *  @param u4Arg      SD argument.
 *  @retval  S_OK  Success.
 *  @retval  Others  Fail.
 */
//-------------------------------------------------------------------------
static INT32 eMsdcSendCmd(UINT32 ch, struct mtk_emsdc_host *host, UINT32 u4Cmd, UINT32 u4Arg)
{
  // Save for check
  host->u4Cmd = u4Cmd;
  host->u4Arg = u4Arg;
  host->u4CmdError = 0;
  
  // Clear Vector variable
//  eMsdcAccuVect = 0;    
  eMsdcClrIntr(ch);
  
  // Set command state flag
  host->sdCmdState = EMSDC_SD_CMD_START;

  // Set SDC Argument
  EMSDC_WRITE32(E_SDC_ARG(ch), u4Arg);

  /* Send the commands to the device */
  EMSDC_WRITE32(E_SDC_CMD(ch), u4Cmd);

  return 0;
}




/*****************************************************************************\
 *                                                                           *
 * Core functions                                                            *
 *                                                                           *
\*****************************************************************************/



static int emsdc_prepare_data(struct mtk_emsdc_host *host, struct emmc_data *data)
{
  int ret = S_OK;
  unsigned int size;

  /*
   * Calculate size.
   */
  size = data->blocks * data->blocksize;

  /*
   * The buffer for DMA is only 64 kB.
   */
  ASSERT(size < BASIC_DMA_MAX_LEN);

  if (size > BASIC_DMA_MAX_LEN)
  {
    ret = E_INVALID_ARG;
    EMSDC_LOG(EMSDC_LOG_ERROR, "DMA size larger then 64KB, size = 0x%x\n", size);
    return ret;
  }
  
  // Setup DMA
  if (data->flags & MMC_DATA_WRITE)
  {
    eMsdcDmaEnable(0, (UINT8*)(data->src), size, TRUE);
  }
  else if (data->flags & MMC_DATA_READ)
  {
    eMsdcDmaEnable(0, (UINT8*)(data->dest), size, FALSE);
  }

  return ret;
}

/*
 * Command handling
 */

static inline void emsdc_get_short_reply(UINT32 ch, struct mtk_emsdc_host *host,
struct emmc_cmd *cmd)
{
  cmd->response[0]  = eMsdcReadResp(ch, 1);
}

static inline void emsdc_get_long_reply(UINT32 ch, struct mtk_emsdc_host *host,
struct emmc_cmd *cmd)
{
  cmd->response[0]  = eMsdcReadResp(ch, 4);  // Bit 127~96
  cmd->response[1]  = eMsdcReadResp(ch, 3);  // Bit 95~64
  cmd->response[2]  = eMsdcReadResp(ch, 2);  // Bit 63~32
  cmd->response[3]  = eMsdcReadResp(ch, 1);  // Bit 31~0
//  EMSDC_LOG(EMSDC_LOG_ERROR, "response[0] = 0x%x\n", cmd->response[0]);
//  EMSDC_LOG(EMSDC_LOG_ERROR, "response[1] = 0x%x\n", cmd->response[1]);
//  EMSDC_LOG(EMSDC_LOG_ERROR, "response[2] = 0x%x\n", cmd->response[2]);
//  EMSDC_LOG(EMSDC_LOG_ERROR, "response[3] = 0x%x\n", cmd->response[3]);
}

static int emsdc_err_2_emmc_err(struct mtk_emsdc_host *host, int emsdc_err)
{
  int err = S_OK;

  switch(emsdc_err)
  {
    case S_OK:
	err = S_OK;
	break;
  
    case E_CMD_TIMEOUT:
    case E_DATA_TIMEOUT:
      err = EMMC_TIMEOUT;
	break;
		
    case E_NO_CARD:
      err = EMMC_NO_CARD_ERR;
	break;	
		
    case E_SD_R3_OCR_BUSY:
    case E_INVALID_CARD:
    case E_INVALID_BLOCK_SIZE:
    case E_INVALID_ARG:
    case E_SD_APPCMD_FAILED:
    case E_RESPBUSY:
    case E_SD_NOT_SUPPORT_WIDE_BUS:
    case E_UNKNOWN_CARD:
    case E_CARD_IS_LOCKED:
    case E_FAIL:
      err = EMMC_COMM_ERR;
	break;	
	
    case E_NO_RESPONSE:
    case E_DATA_CRC_ERROR:
    case E_CMD_RSP_CRC_ERR:
    case E_STATUS:
      err = EMMC_UNUSABLE_ERR;
      break;
  }
  
  host->u4CmdError = err;
  	
  return err;
}

static int eMsdcWaitDataLineIdle (UINT32 ch)
{
  long u4JiffSt = 0, u4JiffEnd = 0, u4DiffTime;

  // Wait Reset done
  u4JiffSt = u4JiffEnd = get_timer(0);
  while (1)
  {
    u4DiffTime = (u4JiffEnd - u4JiffSt) * 1000 / CONFIG_SYS_HZ;   // unit : ms

    if (0 != (EMSDC_READ32(E_MSDC_PS(ch)) & 0x00010000))
    {
       break;
    }
    else if (u4DiffTime >= EMSDC_WAIT_DATA_TIMEOUT_LIMIT)
    {
      EMSDC_LOG(EMSDC_LOG_ERROR, "timeout error !\n");
      return E_ERASE_TIMEOUT;
    }

    u4JiffEnd = get_timer(0);
  }

  return SUCCESS;
}


/*
 * Sends a command out on the bus.  Takes the emmc pointer,
 * a command pointer, and an optional data pointer.
 */
static int emsdc_send_command(struct emmc *emmc, struct emmc_cmd *cmd, struct emmc_data *data)
{
  INT32 i4Ret = 0;
  UINT32 ch = 0;
//  struct emmc_data *data = cmd->data;
  unsigned int sdcCmd = 0;	
  struct mtk_emsdc_host *host = emmc->priv;	 

  ASSERT(host != NULL);
  ASSERT(cmd != NULL);

  host ->u4CmdError = 0;
  
  EMSDC_LOG(EMSDC_LOG_SDHWCMD, "Start - Sending cmd (%d)\n", cmd->cmdidx);


#ifdef CC_CARD_DETECT_PIN
  // handle interrupt status only in card inserted state.
  if (!(emsdc_card_exist()))
  {
    EMSDC_LOG(EMSDC_LOG_ERROR, " No Card error.\n");
    return E_NO_CARD;
  }
#endif

  // Clear command state flag
  host->sdCmdState = EMSDC_SD_CMD_INIT;
  
  // Check if rx/tx fifo is zero
  eMsdcChkFifo(ch);

  // Clear interrupt Vector
  eMsdcClrIntr(ch);    
    
  // Wait SDC bus idle
  if (eMsdcWaitSDCBusIdle(ch) == FAILED)
  {
      EMSDC_LOG(EMSDC_LOG_ERROR, "Wait SDC Bus idle Failed !!, cmd (%d)\n", cmd->cmdidx);
      i4Ret = E_FAIL;
      goto end;
  }  
    
  // Setup command 1 : opcode & response type
  switch (cmd->resp_type)
  {
    case MMC_RSP_NONE:
      sdcCmd |= E_SDC_CMD_RSPTYPE_NO;
      EMSDC_LOG(EMSDC_LOG_SDHWCMD, "Response None\n");
      break;
    case MMC_RSP_R1:
      sdcCmd |= E_SDC_CMD_RSPTYPE_R1;
      EMSDC_LOG(EMSDC_LOG_SDHWCMD, "Response R1\n");
      break;
    case MMC_RSP_R1b:
      sdcCmd |= E_SDC_CMD_RSPTYPE_R1B;
      EMSDC_LOG(EMSDC_LOG_SDHWCMD, "Response R1B\n");
      break;
    case MMC_RSP_R2:
      sdcCmd |= E_SDC_CMD_RSPTYPE_R2;
      EMSDC_LOG(EMSDC_LOG_SDHWCMD, "Response R2\n");
      break;
    case MMC_RSP_R3:
      sdcCmd |= E_SDC_CMD_RSPTYPE_R3;
      EMSDC_LOG(EMSDC_LOG_SDHWCMD, "Response R3\n");
      break;
    default:
   //   BUG_ON(1);
      break;
  }

  // Set SDC_CMD.CMD
  sdcCmd |= (cmd->cmdidx & 0x3F);

  // Setup command 2 : Read/Write flag
  if (data)
  {
	EMSDC_LOG(EMSDC_LOG_SDHWCMD, "RW LBA = 0x%x \n", cmd->cmdarg);
    // Set Blk Length
    sdcCmd |= ((data->blocksize) << E_SDC_CMD_LEN_SHIFT);

    // Set Blk number    
    EMSDC_WRITE32(E_SDC_BLK_NUM(ch), data->blocks);
    EMSDC_LOG(EMSDC_LOG_SDHWCMD, "blocks=%d, blocksize=%d \n", data->blocks, data->blocksize);
    if (data->flags & MMC_DATA_WRITE)
    {	
    //  host->wreqcnt++;

      if (data->blocks > 1)
      {
        sdcCmd |= (E_SDC_CMD_WRITE|E_DTYPE_MULTI_BLK);
      }
      else
      {
        sdcCmd |= (E_SDC_CMD_WRITE|E_DTYPE_SINGLE_BLK);
      }
    }
    else if (data->flags & MMC_DATA_READ)
    {
   //   host->rreqcnt++;
    
      if (data->blocks > 1)
      {
        sdcCmd |= (E_SDC_CMD_READ|E_DTYPE_MULTI_BLK);
      }
      else
      {
        sdcCmd |= (E_SDC_CMD_READ|E_DTYPE_SINGLE_BLK);
      }
    }
  }

  // Setup command 3 : Add stop bit for stop command
  if (cmd->cmdidx== 12)
  {
    sdcCmd |= E_SDC_CMD_STOP;
  }

 // cmd->error = 0;
  
  // Isssue command
  i4Ret = eMsdcSendCmd(ch, host, sdcCmd, cmd->cmdarg);

  /*
   *  Wait for the request to complete.
   */
  // wait command ready.
  i4Ret = eMsdcWaitCmdRdyOrTo(ch, host, cmd);
 
  if (i4Ret == S_OK)
  {
    /*
     * Do we expect a reply?
     */
    if (cmd->resp_type & MMC_RSP_PRESENT)
    {
      if (cmd->resp_type & MMC_RSP_136)
        emsdc_get_long_reply(ch, host, cmd);
      else
        emsdc_get_short_reply(ch, host, cmd);
    }
      
    /*
     * Prepare data before command send
     */
    if (data)
    {
      i4Ret = emsdc_prepare_data(host, data);
      if (i4Ret)
      {
        EMSDC_LOG(EMSDC_LOG_ERROR, "emsdc_prepare_data failed !!\n");
        goto end;
      }     

      i4Ret = eMsdcWaitDataRdyOrTo(ch, host, data);
      if (i4Ret)
      {
        eMsdcErrorHandling(ch, data);
        EMSDC_LOG(EMSDC_LOG_ERROR, "Wait data failed !\n");
        goto end;
      }
	  
	eMsdcDmaDisable(ch);
      eMsdcDmaStatusChk(ch);
  
    }  
    else
    {          
      if(cmd->cmdidx == MMC_CMD_ERASE)
      {
         i4Ret = eMsdcWaitDataLineIdle(ch);
	}
	
      // Set command state flag
      host->sdCmdState = EMSDC_SD_CMD_END;
    }
  }

end:
  emsdc_err_2_emmc_err(host,i4Ret);
  EMSDC_LOG(EMSDC_LOG_SDHWCMD, "Finish - Sent cmd (%d), error: %d, i4Ret: 0x%x\n \n", cmd->cmdidx, host->u4CmdError, i4Ret);

  return host->u4CmdError;
}

/*****************************************************************************\
 *                                                                           *
 * MMC callbacks                                                             *
 *                                                                           *
\*****************************************************************************/




/*------------------------------------------------------------------------------
    EMSDC host driver init/exit entry
------------------------------------------------------------------------------*/
static int emsdc_init(struct emmc *emmc)
{
    struct mtk_emsdc_host *host = &emsdc_host;
    int ch = 0;

    // Reset fci_host
    memset(host, 0, sizeof(struct mtk_emsdc_host));
    
    // set default bus_width
    emmc->bus_width = 1;

    // set initial clock (initial clock will be set in FCI_Init)
    emmc->clock = 1;
    
    // Initialize fci hardware
    return eMsdcInit(ch);
}

static int emsdc_initialize(void)
{
    struct emmc *emmc;
    struct mtk_emsdc_host *host = &emsdc_host;

    // Reset fci_host
    memset(host, 0, sizeof(struct mtk_emsdc_host));
    
    emmc = malloc(sizeof(struct emmc));

    memset(emmc, 0, sizeof(struct emmc));

    sprintf(emmc->name, DRIVER_NAME);
    
    host->chip_id = CONFIG_CHIP_VER_CURR;
    host->base = EMSDC_BASE_ADDR;
      
    emmc->priv = host;
    emmc->send_cmd = emsdc_send_command;
    emmc->set_ios = emsdc_set_ios;
    emmc->init = emsdc_init;

    emmc->voltages |= (MMC_VDD_30_31 | MMC_VDD_31_32 | MMC_VDD_32_33 | 
                               MMC_VDD_33_34 | MMC_VDD_34_35 | MMC_VDD_35_36);

    emmc->host_caps = MMC_MODE_4BIT | MMC_MODE_HS | MMC_MODE_HS_52MHz;

    emmc->f_min = LOWEST_BUS_FREQ;
    emmc->f_max = BUS_CLK_FREQ;    

    emmc_register(emmc);

    printf("[%s] %d", __func__, (int)host->chip_id);
    printf(" At 0x%x config %d pwr %d\n", (int)host->base, _u4eSDMultiFuncCfg, _u4eSDPwrCdPinCfg);

    return 0;
}

int emsdc_mmc_init(void)
{
    return emsdc_initialize();
}


