/********************************************************************************************
 *     LEGAL DISCLAIMER
 *
 *     (Header of MediaTek Software/Firmware Release or Documentation)
 *
 *     BY OPENING OR USING THIS FILE, BUYER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 *     THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE") RECEIVED
 *     FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO BUYER ON AN "AS-IS" BASIS
 *     ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES, EXPRESS OR IMPLIED,
 *     INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *     A PARTICULAR PURPOSE OR NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY
 *     WHATSOEVER WITH RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 *     INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND BUYER AGREES TO LOOK
 *     ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. MEDIATEK SHALL ALSO
 *     NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE RELEASES MADE TO BUYER'S SPECIFICATION
 *     OR TO CONFORM TO A PARTICULAR STANDARD OR OPEN FORUM.
 *
 *     BUYER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND CUMULATIVE LIABILITY WITH
 *     RESPECT TO THE MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION,
 *     TO REVISE OR REPLACE THE MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE
 *     FEES OR SERVICE CHARGE PAID BY BUYER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 *     THE TRANSACTION CONTEMPLATED HEREUNDER SHALL BE CONSTRUED IN ACCORDANCE WITH THE LAWS
 *     OF THE STATE OF CALIFORNIA, USA, EXCLUDING ITS CONFLICT OF LAWS PRINCIPLES.
 ************************************************************************************************/

/** @file msdc_const.h
 *  Macro definition for SD card.
 */

#ifndef EMSDC_CONST_H
#define EMSDC_CONST_H

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// MSDC timeout definitions  (Timer counts by 27Mhz OSC => 1 tick = 37ns)
//---------------------------------------------------------------------------

#define EMSDC_RST_TIMEOUT_LIMIT          20         // MSDC hw reset time < 10 clock => take clk = 400khz as calculation base, 1ms = 400 msdc clock
#define EMSDC_CLK_TIMEOUT_LIMIT          20         // MSDC clk stable time < 10 clock => take clk = 400khz as calculation base, 1ms = 400 msdc clock
#define EMSDC_FIFOCLR_TIMEOUT_LIMIT    20       // MSDC fifoclr reset time < 10 clock => take clk = 400khz as calculation base, 1ms = 400 msdc clock

#define EMSDC_WAIT_SDC_BUS_TIMEOUT_LIMIT                    1000
#define EMSDC_WAIT_CMD_TIMEOUT_LIMIT                           100     // CMD Timeout is fixed value 64 clocks, take clk = 400khz as calculation base, 100ms = 100 * 400 msdc clock
#define EMSDC_WAIT_DATA_TIMEOUT_LIMIT                          5000
#define EMSDC_WAIT_DATA_COMPLETE_TIMEOUT_LIMIT         1000

//---------------------------------------------------------------------------
// MSDC Error code.
//---------------------------------------------------------------------------

#define S_OK                                            0
#define E_CMD_TIMEOUT                       1
#define E_NO_CARD                               2
#define E_SD_R3_OCR_BUSY                3
#define E_INVALID_CARD                       5
#define E_INVALID_BLOCK_SIZE           6 /* over 2048 or not multiple of 4 */
#define E_DATA_CRC_ERROR                 7
#define E_CMD_RSP_CRC_ERR               9
#define E_DATA_TIMEOUT                      10
#define E_STATUS                                    11
#define E_INVALID_ARG                         12     /* Joshua add */
#define E_SD_APPCMD_FAILED              13
#define E_RESPBUSY                                14 /* Joshua add */
#define E_NO_RESPONSE                        15
#define E_SD_NOT_SUPPORT_WIDE_BUS       16
#define E_UNKNOWN_CARD                     20
#define E_MS_CMD_CRC_ERROR             21
#define E_MS_SET_RW_REG_ADRS         22
#define E_ERASE_TIMEOUT                      23

#define E_CARD_IS_LOCKED                    60

#define E_FAIL                                          0xFF

//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------

#define SUPPORT_8_BIT					(0)

#define HOST_OCR						(0x00FF8000)
#define HOST_MMC_OCR          (0x403C0000)
#define HOST_HCS						(1)
#define HOST_VHS						(0x1)

// clock control.
#define SDC_HIGH_CLOCK  (2)
#define SDC_NORM_CLOCK  (1)
#define SDC_LOW_CLOCK   (0)


#define  EMSDC_CLK_54M          ((UINT32)54000000)  
#define  EMSDC_CLK_50M          ((UINT32)50000000)

#define  EMSDC_CLK_36M          ((UINT32)36000000)
#define  EMSDC_CLK_27M          ((UINT32)27000000)
#define  EMSDC_CLK_23M          ((UINT32)23000000)
#define  EMSDC_CLK_13_5M      ((UINT32)13500000)
#define  EMSDC_CLK_3375K       ((UINT32)3375000)
#define  EMSDC_CLK_420K       ((UINT32)420000)
#define  EMSDC_CLK_210K       ((UINT32)210000)

#define INIT_BUS_FREQ			EMSDC_CLK_210K
#define BUS_CLK_FREQ			EMSDC_CLK_36M    // max freq of work
#define SRC_CLK_FREQ			EMSDC_CLK_27M    // max freq of HW support
#define LOWEST_BUS_FREQ		EMSDC_CLK_210K

#define DMA_BST_8						(3)
#define DMA_BST_16						(4)
#define DMA_BST_32						(5)
#define DMA_BST_64						(6)
#define BASIC_DMA_MAX_LEN				(0xFFFF)
#define DESCRIPTER_DMA_MAX_LEN		(0x10000)

#define GPD_MAX_LEN						(0xFFFF)
#define BD_MAX_LEN						(0xFFFF)
#define BD_MAX_NUM						(128)
#define TRANS_MAX_BLK_NUM				(128)


#define MMC_DEF_RCA     0x0001

#define SD_DEF_DSR      0x0404

#define SDC_OCR_SD_DEFAULT     0x00FC0000   // for SD 1.X
#define SDC_OCR_HD_DEFAULT     0x40FC0000   // for SD 2.0

#define SDC_HVS_DEFAULT     0x000001AA

#define SDC_CMD6_DEFAULT_SPEED    ((UINT32)0x00FFFF00)
#define SDC_CMD6_HIGH_SPEED       ((UINT32)0x00FFFF01)
#define SDC_CMD6_CHECK_FUNC 		((UINT32)0x00 << 31)
#define SDC_CMD6_SWITCH_FUNC 	   ((UINT32)0x01 << 31)

// MMC CMD6 Argument :
// (1) Bus Width Selection :
// Access bits = 0x03 (Write byte), Index = 0xB7 = 183, Value = 0(1bit), 1(4bits), 2(8bits)
#define MMC_CMD6_ARG_1BIT_BUS  0x03B70000
#define MMC_CMD6_ARG_4BIT_BUS 0x03B70100
#define MMC_CMD6_ARG_8BIT_BUS 0x03B70200

// (2) High SPeed Mode Selection :
// Access bits = 0x03 (Write byte), Index = 0xB9 = 185, Value = 0(26 Mhz max), 1(52 Mhz max)
#define MMC_CMD6_ARG_NORM_SPEED  0x03B90000
#define MMC_CMD6_ARG_HIGH_SPEED   0x03B90100

// #define SDC_OCR_DEFAULT      0x00020000
#define TRY_OCR             10
#define SDC_OCR_BUSY        0x80000000
#define SDC_RCA_DEFAULT     0x0000
#define ARG_RCA_MASK        0xffff0000
#define SDC_RCA_MMC         1
//#define SDC_DSR_DEFAULT       0x0404
#define SDC_NO_ARG          0
#define SDC_MAX_BKLENGTH    2048
#define SDC_MAX_LOCKBK      34

//masks for command error response(32) of SDC_CSTA
#define SDC_OUT_OF_RANGE        0x80000000
#define SDC_ADDRESS_ERROR       0x40000000
#define SDC_BLOCK_LEN_ERROR     0X20000000
#define SDC_ERASE_SEQ_ERROR     0X10000000
#define SDC_ERASE_PARAM         0X08000000
#define SDC_WP_VIOLATION        0X04000000
#define SDC_CARD_IS_LOCKED      0x02000000
#define SDC_LOCK_UNLOCK_FAILED  0X01000000
#define SDC_COM_CRC_ERROR       0X00800000
#define SDC_ILLEGAL_COMMAND     0X00400000
#define SDC_CARD_ECC_FAILED     0X00200000
#define SDC_CC_ERROR            0X00100000
#define SDC_ERROR               0X00080000
#define SDC_UNDERRUN            0X00040000
#define SDC_OVERRUN             0X00020000
#define SDC_CIDCSD_OVERWRITE    0X00010000
#define SDC_AKE_SEQ_ERROR       0X00000008

#define SDC_CSTA_MASK 0xffff0000

//masks for card status contained in R1
#define R1_AKE_SEQ_ERROR_3          0x00000008
#define R1_APP_CMD_5                0x00000020
#define R1_READY_FOR_DATA_8         0x00000100
#define R1_CURRENT_STATE_9_12       0x00001e00
#define R1_ERASE_RESET_13           0x00002000
#define R1_CARD_ECC_DISABLE_14      0x00004000
#define R1_WP_ERASE_SKIP_15         0x00008000
#define R1_CID_CSD_OVERWRITE_16     0x00010000
#define R1_OVERRUN_17               0x00020000
#define R1_UNDERRUN_18              0x00040000
#define R1_ERROR_19                 0x00080000
#define R1_CC_ERROR_20              0x00100000
#define R1_CARD_ECC_FAILED_21       0x00200000
#define R1_ILLEGAL_COMMAND_22       0x00400000
#define R1_COM_CRC_ERROR_23         0x00800000
#define R1_LOCK_UNLOCK_FAILED_24    0x01000000
#define R1_CARD_IS_LOCKED_25        0x02000000
#define R1_WP_VIOLATION_26          0x04000000
#define R1_ERASE_PARAM_27           0x08000000
#define R1_ERASE_SEQ_ERROR_28       0x10000000
#define R1_BLOCK_LEN_ERROR_29       0x20000000
#define R1_ADDRESS_ERROR_30         0x40000000
#define R1_OUT_OF_RANGE_31          0x80000000

#define SD_RW_ERRORS    (R1_OUT_OF_RANGE_31|R1_ADDRESS_ERROR_30|R1_ERASE_PARAM_27|R1_WP_VIOLATION_26|R1_CARD_IS_LOCKED_25|R1_LOCK_UNLOCK_FAILED_24 \
                        |R1_COM_CRC_ERROR_23|R1_ILLEGAL_COMMAND_22|R1_CARD_ECC_FAILED_21|R1_CC_ERROR_20|R1_ERROR_19                                \
                        |R1_UNDERRUN_18|R1_OVERRUN_17|R1_CID_CSD_OVERWRITE_16|R1_WP_ERASE_SKIP_15|R1_CARD_ECC_DISABLE_14                           \
                        |R1_ERASE_RESET_13)
#define SD_COM_ERRORS   (R1_OUT_OF_RANGE_31|R1_ERASE_PARAM_27|R1_WP_VIOLATION_26|R1_CARD_IS_LOCKED_25|R1_LOCK_UNLOCK_FAILED_24                     \
                        |R1_COM_CRC_ERROR_23|R1_ILLEGAL_COMMAND_22|R1_CARD_ECC_FAILED_21|R1_CC_ERROR_20|R1_ERROR_19                                \
                        |R1_UNDERRUN_18|R1_OVERRUN_17|R1_CID_CSD_OVERWRITE_16|R1_WP_ERASE_SKIP_15|R1_CARD_ECC_DISABLE_14                           \
                        |R1_ERASE_RESET_13)



//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------

#define SUCCESS									(0)
#define FAILED									   (-1)

//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------

#endif // SDC_CONST_H
